/**
 *******************************************************************************
 * @file    txz_fuart_mdma.c
 * @brief   This file provides API functions for FUART driver.
 * @brief   If MDMA transfer is used, it is necessary.
 * @version V1.0.0.0
 * $Date:: 2017-08-28 10:52:31 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_fuart_mdma.h"

#if defined(__FUART_MDMA_H)

/**
 *  @addtogroup Periph_Driver
 *  @{
 */

/** 
 *  @addtogroup FUART
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup FUART_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group FUART_Private_define */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup FUART_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group FUART_Private_define */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup FUART_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group FUART_Private_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup FUART_Private_typedef
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group FUART_Private_typedef */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup FUART_Private_fuctions
 *  @{
 */
static void tx_transfer_done(uint32_t id, TXZ_Result result);
static void rx_transfer_done(uint32_t id, TXZ_Result result);
/*--------------------------------------------------*/
/**
  * @brief     MDMA TX Transfer Done Handler.
  * @param     ch     :MDMA channel.
  * @param     result :Result.
  * @retval    -
  * @note      This Function is called by DMA driver.
  */
/*--------------------------------------------------*/
static void tx_transfer_done(uint32_t id, TXZ_Result result)
{
    fuart_mdma_t *p_obj = (fuart_mdma_t *)id;
    if(result == TXZ_SUCCESS)
    {
        /* FUART Transmit Handler Call. */
        if ((p_obj != FUART_NULL) && (p_obj->super.transmit.handler != FUART_NULL))
        {
            p_obj->super.transmit.handler(p_obj->super.init.id, result);
        }
    }
}

/*--------------------------------------------------*/
/**
  * @brief     MDMA RX Transfer Done Handler.
  * @param     ch     :MDMA channel.
  * @param     result :Result.
  * @retval    -
  * @note      This Function is called by DMA driver.
  */
/*--------------------------------------------------*/
static void rx_transfer_done(uint32_t id, TXZ_Result result)
{
    fuart_mdma_t *p_obj = (fuart_mdma_t *)id;

    if(result == TXZ_SUCCESS)
    {
        /* FUART Transmit Handler Call. */
        if ((p_obj != FUART_NULL) && (p_obj->super.receive.handler != FUART_NULL))
        {
            p_obj->super.receive.handler(p_obj->super.init.id, result, &p_obj->super.receive.info);
        }
    }
}

/**
 *  @}
 */ /* End of group FUART_Private_functions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup FUART_Exported_functions
 *  @{
 */
/*--------------------------------------------------*/
/**
  * @brief     Initialize the FUART MDMA object.
  * @param     p_obj   :FUART MDMA object.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result fuart_mdma_init(fuart_mdma_t *p_obj)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    /*------------------------------*/
    /*  Construct Superclass        */
    /*------------------------------*/
    if (fuart_init(&p_obj->super) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }
    /*------------------------------*/
    /*  Construct Subclass          */
    /*------------------------------*/
    if (result == TXZ_SUCCESS)
    {
        __IO uint32_t cr1 = p_obj->super.p_instance->DMACR;
        
        /*--- DMA Driver "Transmit Channel" ---*/
        if (p_obj->init.tx.type != FUART_MDMA_TYPE_UNUSE)
        {
           TSB_FURT_TypeDef *fuart_reg = p_obj->super.p_instance;
           uint32_t ch = 0;
           uint32_t reg = 0;
           
            cr1 &= ~FUARTxDMACR_TXDMAE_MASK;
            cr1 |=  FUARTxDMACR_TXDMAE_ENABLE;
            
            if(fuart_reg == TSB_FURT0)
            {
                ch = 15;
                reg = TSB_TSEL0->CR3;
                reg &= 0x00FFFFFF;
                reg |= 0x23000000;
                TSB_TSEL0->CR3 = reg;    /* INSEL15: MDMA CH15: FUART CH0 TX: FUART0TXDMASREQ */
            }
            else if(fuart_reg == TSB_FURT1)
            {
                ch = 3;
                reg = TSB_TSEL0->CR0;
                reg &= 0x00FFFFFF;
                reg |= 0x13000000;
                TSB_TSEL0->CR0 = reg;    /* INSEL3: MDMA CH3: FUART CH1 TX: FUART1TXDMASREQ */
            }
            else
            {
                result = TXZ_ERROR;
            }
            if(result == TXZ_SUCCESS)
            {
                mdma_ch_interrupt_setting_t int_ch;
                
                p_obj->init.tx.ch = ch;
                int_ch.priority = MDMA_PRIORITY_HIGH;
                int_ch.id = (uint32_t)p_obj;
                int_ch.handler = tx_transfer_done;
                if (mdma_ch_init(ch, int_ch) != TXZ_SUCCESS)
                {
                    result = TXZ_ERROR;
                }
            }
        }
        /*--- DMA Driver "Receive Channel" ---*/
        if (p_obj->init.rx.type != FUART_MDMA_TYPE_UNUSE)
        {
            TSB_FURT_TypeDef *fuart_reg = p_obj->super.p_instance;
            uint32_t ch = 0;
            uint32_t reg = 0;
            
            cr1 &= ~FUARTxDMACR_RXDMAE_MASK;
            cr1 |=  FUARTxDMACR_RXDMAE_ENABLE;
            if(fuart_reg == TSB_FURT0)
            {
                ch = 17;
                reg = TSB_TSEL0->CR4;
                reg &= 0xFFFF00FF;
                reg |= 0x00002300;
                TSB_TSEL0->CR4 = reg;    /* INSEL17: MDMA CH17: FUART CH0 RX: FUART0RXDMASREQ */
            }
            else if(fuart_reg == TSB_FURT1)
            {
                ch = 2;
                reg = TSB_TSEL0->CR0;
                reg &= 0xFF00FFFF;
                reg |= 0x00130000;
                TSB_TSEL0->CR0 = reg;    /* INSEL2: MDMA CH2: FUART CH1 RX: FUART1RXDMASREQ */
            }
            else
            {
                result = TXZ_ERROR;
            }
            if(result == TXZ_SUCCESS)
            {
                mdma_ch_interrupt_setting_t int_ch;
                
                p_obj->init.rx.ch = ch;
                int_ch.priority = MDMA_PRIORITY_HIGH;
                int_ch.id = (uint32_t)p_obj;
                int_ch.handler = rx_transfer_done;
                if (mdma_ch_init(ch, int_ch) != TXZ_SUCCESS)
                {
                    result = TXZ_ERROR;
                }
            }
        
        }
        if(result == TXZ_SUCCESS){
            /*--- FUARTxCR1 ---*/
            p_obj->super.p_instance->DMACR = cr1;
        }
    }

    return (result);
}
/*--------------------------------------------------*/
/**
  * @brief     Release the FUART MDMA object.
  * @param     p_obj   :FUART MDMA object.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result fuart_mdma_deinit(fuart_mdma_t *p_obj)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    /*------------------------------*/
    /*  Destruct Subclass           */
    /*------------------------------*/
    /*--- DMA Driver "Transmit Channel" ---*/
    if (p_obj->init.tx.type != FUART_MDMA_TYPE_UNUSE)
    {
        __IO uint32_t cr1 = p_obj->super.p_instance->DMACR;
        TSB_FURT_TypeDef *fuart_reg = p_obj->super.p_instance;
        uint32_t ch = 0;

        /*--- FUARTxCR1   ---*/
        cr1 &= ~FUARTxDMACR_TXDMAE_MASK;
        cr1 |=  FUARTxDMACR_TXDMAE_DISABLE;
        p_obj->super.p_instance->DMACR = cr1;
        /*--- DMA Driver ---*/
        if(fuart_reg == TSB_FURT0)
        {
            ch = 15;
            TSB_TSEL0->CR3  &= 0x00FFFFFF;              /* INSEL15: MDMA CH15: FUART CH0 TX: FUART0TXDMASREQ */
        }
        else if(fuart_reg == TSB_FURT1)
        {
            ch = 3;
            TSB_TSEL0->CR0 &= 0x00FFFFFF;                /* INSEL3: MDMA CH3: FUART CH1 TX: FUART1TXDMASREQ */
        }
        else
        {
            result = TXZ_ERROR;
        }
        if(result == TXZ_SUCCESS)
        {
            if (mdma_ch_deinit(ch) != TXZ_SUCCESS)
            {
                result = TXZ_ERROR;
            }
        }
    }
    /*--- DMA Driver "Receive Channel" ---*/
    if (p_obj->init.rx.type != FUART_MDMA_TYPE_UNUSE)
    {
        __IO uint32_t cr1 = p_obj->super.p_instance->DMACR;
        TSB_FURT_TypeDef *fuart_reg = p_obj->super.p_instance;
        uint32_t ch = 0;

        /*--- FUARTxCR1   ---*/
        cr1 &= ~FUARTxDMACR_RXDMAE_MASK;
        cr1 |=  FUARTxDMACR_RXDMAE_DISABLE;
        p_obj->super.p_instance->DMACR = cr1;
        /*--- DMA Driver ---*/
        if(fuart_reg == TSB_FURT0)
        {
            ch = 17;
            TSB_TSEL0->CR4 &= 0xFFFF00FF;                /* INSEL17: MDMA CH17: FUART CH0 RX: FUART0RXDMASREQ */
        }
        else if(fuart_reg == TSB_FURT1)
        {
            ch = 2;
            TSB_TSEL0->CR0 &= 0xFF00FFFF;                /* INSEL2: MDMA CH2: FUART CH1 RX: FUART1RXDMASREQ */
        }
        else
        {
            result = TXZ_ERROR;
        }
        if(result == TXZ_SUCCESS){
            if (mdma_ch_deinit(ch) != TXZ_SUCCESS)
            {
                result = TXZ_ERROR;
            }
        }
    }
    /*------------------------------*/
    /*  Destruct Superclass         */
    /*------------------------------*/
    if (fuart_deinit(&p_obj->super) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }

    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Discard transmit.
  * @param     p_obj   :FUART MDMA object.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      This function clears transmit's fifo, end flag and error info.
  * @attention This function is not available in interrupt.
  * @attention Use after interrupt is disabled.
  */
/*--------------------------------------------------*/
TXZ_Result fuart_mdma_discard_transmit(fuart_mdma_t *p_obj)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    /* Check the FUART_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef DEBUG */
    /*------------------------------*/
    /*  Stop DMA                    */
    /*------------------------------*/
    if (p_obj->init.tx.type != FUART_MDMA_TYPE_UNUSE)
    {
        if ( mdma_stop_ch(p_obj->init.tx.ch) != TXZ_SUCCESS)
        {
            result = TXZ_ERROR;
        }
    }
    /*------------------------------*/
    /*  Use Super Class             */
    /*------------------------------*/
    if (fuart_discard_transmit(&p_obj->super) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }

    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Discard receive.
  * @param     p_obj   :FUART MDMA object.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      This function clears receive's fifo, end flag and error info.
  * @attention This function is not available in interrupt.
  * @attention Use after interrupt is disabled.
  */
/*--------------------------------------------------*/
TXZ_Result fuart_mdma_discard_receive(fuart_mdma_t *p_obj)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    /* Check the FUART_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef DEBUG */
    /*------------------------------*/
    /*  Stop DMA                    */
    /*------------------------------*/
    if (p_obj->init.rx.type != FUART_MDMA_TYPE_UNUSE)
    {
        if ( mdma_stop_ch(p_obj->init.rx.ch) != TXZ_SUCCESS)
        {
            result = TXZ_ERROR;
        }
    }
    /*------------------------------*/
    /*  Use Super Class             */
    /*------------------------------*/
    if (fuart_discard_receive(&p_obj->super) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }

    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief  Transmit data. Non-Blocking Communication.
  * @param  p_obj   :FUART MDMA object.
  * @param  p_info  :The information of transmit data.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   Asynchronous Processing.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result fuart_mdma_transmitIt(fuart_mdma_t *p_obj, fuart_transmit_t *p_info)
{
    TXZ_Result result  = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    /*------------------------------*/
    /*  Disable Transmit            */
    /*------------------------------*/
    /*--- FUARTxTRANS ---*/
    /* Write to TXE(=0).               */
    /* Bitband Access.                 */
    if(fuart_mdma_TXE_disable(p_obj) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
        return(result);
    }
    /*------------------------------*/
    /*  DMA Setting                 */
    /*------------------------------*/
    {
    	mdma_descriptor_set_t desc_set_param;
    	uint32_t ch = p_obj->init.tx.ch;
    	uint32_t ttype = p_obj->init.tx.type;
    	mdma_descriptor_set_t *desc_ptr = &desc_set_param;
    	uint32_t desc_num = 0;
    	uint32_t infini = 0;
    	
    	p_obj->super.transmit.info.p_data  = p_info->p_data;
    	p_obj->super.transmit.info.num     = p_info->num;
    	if(ttype == FUART_MDMA_TYPE_AREA_TO_FIX)
    	{
    		desc_ptr->dmode = MDMA_DMODE_INC;
    		desc_ptr->umode = MDMA_UMODE_TRANS_UNI;
    		desc_ptr->usize = MDMA_USIZE_1BYTE;
    		desc_ptr->ttype = MDMA_TTYPE_AREA_FIX;
    		desc_ptr->xfsad = (uint32_t)p_info->p_data;
    		desc_ptr->xfdad = (uint32_t)&p_obj->super.p_instance->DR;;
    		desc_ptr->xfsiz = p_info->num;
    		desc_ptr->dsads = (uint32_t)FUART_NULL;
    		if(mdma_desc_ch_configure(ch, desc_ptr, desc_num, infini) != TXZ_SUCCESS)
            {
                result = TXZ_ERROR;
            }
    		if(mdma_basic_start_ch(ch, MDMA_INTERRUPT_ENABLE) != TXZ_SUCCESS)
            {
                result = TXZ_ERROR;
            }
    	}
    	else
    	{
    		result  = TXZ_ERROR;
    		
    	}
    
    }
    /*------------------------------*/
    /*  Enable Transmit             */
    /*------------------------------*/
    /*--- FUARTxTRANS ---*/
    /* Write to TXE(=1).               */
    /* Bitband Access.                 */
    if(fuart_mdma_TXE_enable(p_obj) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }

    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief  Receive data. Non-Blocking Communication.
  * @param  p_obj   :FUART DMA object.
  * @param  p_info  :The information of receive buffer.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   Asynchronous Processing.
  * @attention  "p_info->rx8(or rx16).num" must be over FIFO max(Refer @ref FUART_FifoMax) num.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result fuart_mdma_receiveIt(fuart_mdma_t *p_obj, fuart_receive_t *p_info)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    /*------------------------------*/
    /*  Disable Receive             */
    /*------------------------------*/
    /* Write to RXE(=0).               */
    /* Bitband Access.                 */
    if(fuart_mdma_RXE_disable(p_obj) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
        return(result);
    }
    /*------------------------------*/
    /*  DMA Setting                 */
    /*------------------------------*/
    {
    	mdma_descriptor_set_t desc_set_param;
    	uint32_t ch = p_obj->init.rx.ch;
    	uint32_t ttype = p_obj->init.rx.type;
    	mdma_descriptor_set_t *desc_ptr = &desc_set_param;
    	uint32_t desc_num = 0;
    	uint32_t infini = 0;
    
    	p_obj->super.receive.info.p_data  = p_info->p_data;
    	p_obj->super.receive.info.num     = p_info->num;
    	if(ttype == FUART_MDMA_TYPE_FIX_TO_AREA)
    	{
    		desc_ptr->dmode = MDMA_DMODE_INC;
    		desc_ptr->umode = MDMA_UMODE_TRANS_UNI;
    		desc_ptr->usize = MDMA_USIZE_1BYTE;
    		desc_ptr->ttype = MDMA_TTYPE_FIX_AREA;
    		desc_ptr->xfsad = (uint32_t)&p_obj->super.p_instance->DR;
    		desc_ptr->xfdad = (uint32_t)p_info->p_data;
    		desc_ptr->xfsiz = p_info->num;
    		desc_ptr->dsads = (uint32_t)FUART_NULL;
    		if(mdma_desc_ch_configure(ch, desc_ptr, desc_num, infini) != TXZ_SUCCESS)
            {
                result = TXZ_ERROR;
            }
    		if(mdma_basic_start_ch(ch, MDMA_INTERRUPT_ENABLE) != TXZ_SUCCESS)
            {
                result = TXZ_ERROR;
            }
    	}
    	else
    	{
    		result  = TXZ_ERROR;
    		
    	}
    }
    /*------------------------------*/
    /*  Enable Receive              */
    /*------------------------------*/
    /*--- FUARTxTRANS ---*/
    /* Write to RXE(=1).               */
    /* Bitband Access.                 */
    if(fuart_mdma_RXE_enable(p_obj) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Uart ch to MDMA ch
  * @param  fuart_ch :Uart channel.
  * @param  tx_ch :MDMA fuart tx channel.
  * @param  rx_ch :MDMA fuart rx channel.
  * @return -
  * @retval -
  * @note   -
  * @attention -
  */
/*--------------------------------------------------*/
TXZ_Result fuart_mdma_get_fuart_ch_to_mdma_ch(uint32_t fuart_ch, uint32_t *tx_ch, uint32_t *rx_ch)
{
    TXZ_Result result = TXZ_SUCCESS;
    
    switch(fuart_ch)
    {
    case FUART_MDMA_FUART_CH0:
        *tx_ch = 15;
        *rx_ch = 17;
        break;
    case FUART_MDMA_FUART_CH1:
        *tx_ch = 3;
        *rx_ch = 2;
        break;
    default:
        result = TXZ_ERROR;
        break;
    }
    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     TXE disable.
  * @param     p_obj   :FUART MDMA object.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  */
/*--------------------------------------------------*/
TXZ_Result fuart_mdma_TXE_disable(fuart_mdma_t *p_obj)
{
    TXZ_Result result  = TXZ_SUCCESS;

    __IO uint32_t cr1 = p_obj->super.p_instance->DMACR;
    
    cr1 &= ~FUARTxDMACR_TXDMAE_MASK;
    cr1 |=  FUARTxDMACR_TXDMAE_DISABLE;
    p_obj->super.p_instance->DMACR = cr1;
    /*------------------------------*/
    /*  Disable Transmit            */
    /*------------------------------*/
    /*--- FUARTxTRANS ---*/
    /* Write to TXE(=0).               */
    /* Bitband Access.                 */
    disable_FUARTxCR_TXE(p_obj->super.p_instance);
    return (result);
}
/*--------------------------------------------------*/
/**
  * @brief     TXE enable.
  * @param     p_obj   :FUART MDMA object.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  */
/*--------------------------------------------------*/
TXZ_Result fuart_mdma_TXE_enable(fuart_mdma_t *p_obj)
{
    TXZ_Result result  = TXZ_SUCCESS;

    __IO uint32_t cr1 = p_obj->super.p_instance->DMACR;

    cr1 &= ~FUARTxDMACR_TXDMAE_MASK;
    cr1 |=  FUARTxDMACR_TXDMAE_ENABLE;
    p_obj->super.p_instance->DMACR = cr1;
    /*------------------------------*/
    /*  Enable Transmit            */
    /*------------------------------*/
    /*--- FUARTxTRANS ---*/
    /* Write to TXE(=0).               */
    /* Bitband Access.                 */
    enable_FUARTxCR_TXE(p_obj->super.p_instance);
    return (result);
}
/*--------------------------------------------------*/
/**
  * @brief     RXE disable.
  * @param     p_obj   :FUART MDMA object.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  */
/*--------------------------------------------------*/
TXZ_Result fuart_mdma_RXE_disable(fuart_mdma_t *p_obj)
{
    TXZ_Result result  = TXZ_SUCCESS;
    __IO uint32_t cr1 = p_obj->super.p_instance->DMACR;
     
    cr1 &= ~FUARTxDMACR_RXDMAE_MASK;
    cr1 |=  FUARTxDMACR_RXDMAE_DISABLE;
    p_obj->super.p_instance->DMACR = cr1;

    /*------------------------------*/
    /*  Disable Transmit            */
    /*------------------------------*/
    /*--- FUARTxTRANS ---*/
    /* Write to RXE(=0).               */
    /* Bitband Access.                 */
    disable_FUARTxCR_RXE(p_obj->super.p_instance);
    return (result);
}
/*--------------------------------------------------*/
/**
  * @brief     RXE enable.
  * @param     p_obj   :FUART MDMA object.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  */
/*--------------------------------------------------*/
TXZ_Result fuart_mdma_RXE_enable(fuart_mdma_t *p_obj)
{
    TXZ_Result result  = TXZ_SUCCESS;
    __IO uint32_t cr1 = p_obj->super.p_instance->DMACR;

    cr1 &= ~FUARTxDMACR_RXDMAE_MASK;
    cr1 |=  FUARTxDMACR_RXDMAE_ENABLE;
    p_obj->super.p_instance->DMACR = cr1;
    /*------------------------------*/
    /*  Enable Transmit            */
    /*------------------------------*/
    /*--- FUARTxTRANS ---*/
    /* Write to RXE(=1).               */
    /* Bitband Access.                 */
    enable_FUARTxCR_RXE(p_obj->super.p_instance);
    return (result);
}

/**
 *  @}
 */ /* End of group FUART_Exported_functions */

/**
 *  @} 
 */ /* End of group FUART */

/**
 *  @} 
 */ /* End of group Periph_Driver */

#endif /* defined(__UART_MDMA_H)  */

#ifdef __cplusplus
}
#endif /* __cplusplus */

