/** 
 *******************************************************************************
 * @file    txz_i2c_api_dma.c
 * @brief   This file provides API functions for I2C driver.
 * @version V0.0.0.1
 * $Date:: 2016-11-25 00:00:00 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_i2c_api_dma.h"

#if defined(__I2C_API_DMA_H)

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @addtogroup UTILITIES
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UTILITIES_Private_macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group UTILITIES_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UTILITIES_Private_define
 *  @{
 */
/** 
 *  @name  Parameter Result
 *  @brief Whether the parameter is specified or not.
 *  @{
 */
#define I2C_DMA_PARAM_OK            ((int32_t)1)              /*!< Parameter is valid(specified).         */
#define I2C_DMA_PARAM_NG            ((int32_t)0)              /*!< Parameter is invalid(not specified).   */
/**
 *  @}
 */ /* End of name Parameter Result */
/** 
 *  @}
 */ /* End of group UTILITIES_Private_define */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UTILITIES_Private_define
 *  @{
 */

/* no define */
     
/** 
 *  @}
 */ /* End of group UTILITIES_Private_define */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UTILITIES_Private_define
 *  @{
 */
 
/*----------------------------------*/
/** 
 * @brief  DMA Transfer Event.
*/
/*----------------------------------*/
enum 
{
    EVENT_NON = 0U,                     /*!< No Event.               */
    EVENT_DONE,                         /*!< Done Event.             */
    EVENT_ERR                           /*!< Error Event.            */
} I2CDMAEvent;

/** 
 *  @}
 */ /* End of group UTILITIES_Private_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UTILITIES_Private_typedef
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group UTILITIES_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UTILITIES_Private_variables
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group UTILITIES_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UTILITIES_Private_functions
 *  @{
 */
#ifdef DEBUG
__STATIC_INLINE int32_t check_param_address(int32_t address);
__STATIC_INLINE int32_t check_param_length(int32_t length);
#endif
static TXZ_Result init_dma_driver(i2c_dma_t *p_obj);
static void discard_receive(i2c_dma_t *p_obj);
static void discard_transmit(i2c_dma_t *p_obj);
static void rx_transfer_done(uint32_t id, TXZ_Result result);
static void tx_transfer_done(uint32_t id, TXZ_Result result);
static void i2c_irq_handler(i2c_dma_t *p_obj);
static void i2c_slave_irq_handler(i2c_dma_t *p_obj);
/* for specification extension */
extern void i2c_enable_irq(i2c_t *p_obj);
extern void i2c_disable_irq(i2c_t *p_obj);

#ifdef DEBUG
/*--------------------------------------------------*/
/** 
  * @brief  Compare the Slave address's parameter.
  * @param  address      :Address.
  * @retval I2C_DMA_PARAM_OK :Available.
  * @retval I2C_DMA_PARAM_NG :Not Available.
  * @note   Here, 10bit address has not supported.
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_address(int32_t address)
{
    int32_t result = I2C_DMA_PARAM_NG;

    if ((address >= 0) && (address <= 255))
    {
        result = I2C_DMA_PARAM_OK;
    }
    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Compare the length's parameter.
  * @param  length        :Length.
  * @retval I2C_DMA_PARAM_OK :Available.
  * @retval I2C_DMA_PARAM_NG :Not Available.
  * @note   This value is DMA upper limit.
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_length(int32_t length)
{
    int32_t result = I2C_DMA_PARAM_NG;

    if ((length > 0) && (length <= 1024))
    {
        result = I2C_DMA_PARAM_OK;
    }
    return (result);
}
#endif

/*--------------------------------------------------*/
/**
  * @brief     Discard receive.
  * @param     p_obj     :i2c DMA object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
static void discard_receive(i2c_dma_t *p_obj)
{
    if (p_obj->init.dma.rx == TXZ_ENABLE)
    {
        if (dma_normal_stopIt(p_obj->init.dma.p_obj, p_obj->info.rxch) != TXZ_SUCCESS)
        {
        }
    }
}

/*--------------------------------------------------*/
/**
  * @brief     Discard transmit.
  * @param     p_obj     :i2c DMA object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
static void discard_transmit(i2c_dma_t *p_obj)
{
    if (p_obj->init.dma.tx == TXZ_ENABLE)
    {
        if (dma_normal_stopIt(p_obj->init.dma.p_obj, p_obj->info.txch) != TXZ_SUCCESS)
        {
        }
    }
}

/*--------------------------------------------------*/
/**
  * @brief     Initialize DMA Driver
  * @param     p_obj     :i2c DMA object.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  */
/*--------------------------------------------------*/
static TXZ_Result init_dma_driver(i2c_dma_t *p_obj)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t rxch;
    uint32_t txch;

    if (p_obj->super.i2c.p_instance == TSB_I2C0)
    {
        rxch = 4;
        txch = 5;
    }
    else if (p_obj->super.i2c.p_instance == TSB_I2C1)
    {
        rxch = 6;
        txch = 7;
    }
    else if (p_obj->super.i2c.p_instance == TSB_I2C2)
    {
        rxch = 8;
        txch = 9;
    }
    else
    {
        rxch = 4; /* no use */
        txch = 5; /* no use */
        result = TXZ_ERROR;
    }
    if (result == TXZ_SUCCESS)
    {
        /* Initialize internal information */
        p_obj->info.rxch  = rxch;
        p_obj->info.txch  = txch;
        p_obj->info.event = EVENT_NON;

        /*--- DMA Driver "Receive Channel" ---*/
        if (p_obj->init.dma.rx == TXZ_ENABLE)
        {
            dma_ch_setting_t *p_dma = &p_obj->init.dma.p_obj->ch[p_obj->info.rxch];

            p_dma->handler       = rx_transfer_done;
            p_dma->id            = (uint32_t)p_obj;
            p_dma->priority      = DMA_PRIORITY_NORMAL;
            if (dma_ch_init(p_obj->init.dma.p_obj, p_obj->info.rxch) != TXZ_SUCCESS)
            {
                result = TXZ_ERROR;
            }
        }
        /*--- DMA Driver "Transmit Channel" ---*/
        if (p_obj->init.dma.tx == TXZ_ENABLE)
        {
            dma_ch_setting_t *p_dma = &p_obj->init.dma.p_obj->ch[p_obj->info.txch];

            p_dma->handler       = tx_transfer_done;
            p_dma->id            = (uint32_t)p_obj;
            p_dma->priority      = DMA_PRIORITY_NORMAL;
            if (dma_ch_init(p_obj->init.dma.p_obj, p_obj->info.txch) != TXZ_SUCCESS)
            {
                result = TXZ_ERROR;
            }
        }
    }
    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     DMA RX Transfer Done Handler.
  * @param     id     :User ID(i2c DMA object).
  * @param     result :Result.
  * @retval    -
  * @note      This Function is called by DMA driver.
  */
/*--------------------------------------------------*/
static void rx_transfer_done(uint32_t id, TXZ_Result result)
{
    i2c_dma_t *p_obj = (i2c_dma_t *)id;
    
    if (p_obj != I2C_NULL)
    {
        if (result == TXZ_SUCCESS)
        {
            p_obj->info.event = EVENT_DONE;
        }
        else
        {
            p_obj->info.event = EVENT_ERR;
        }
        /* Disable intdma, and enable only inti2c. */
        I2C_enable_interrupt(&p_obj->super.i2c);
    }
}

/*--------------------------------------------------*/
/**
  * @brief     DMA TX Transfer Done Handler.
  * @param     id     :User ID(i2c DMA object).
  * @param     result :Result.
  * @retval    -
  * @note      This Function is called by DMA driver.
  */
/*--------------------------------------------------*/
static void tx_transfer_done(uint32_t id, TXZ_Result result)
{
    i2c_dma_t *p_obj = (i2c_dma_t *)id;

    if (p_obj != I2C_NULL)
    {
        if (result == TXZ_SUCCESS)
        {
            p_obj->info.event = EVENT_DONE;
        }
        else
        {
            p_obj->info.event = EVENT_ERR;
        }
        /* Disable intdma, and enable only inti2c. */
        I2C_enable_interrupt(&p_obj->super.i2c);
    }
}

/*--------------------------------------------------*/
/**
  * @brief     I2C Transfer handler (for master)
  * @param     p_obj     :i2c DMA object.
  * @retval    -
  * @note      Implement as INTI2C IRQ handler.
  */
/*--------------------------------------------------*/
static void i2c_irq_handler(i2c_dma_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef DEBUG */

    I2C_clear_int_status(&p_obj->super.i2c);

    if (!I2C_master(&p_obj->super.i2c) || (p_obj->info.event != EVENT_DONE))
    {
        p_obj->super.info.asynch.event = I2C_EVENT_ERROR;
        return;
    }
    if (I2C_transmitter(&p_obj->super.i2c))
    {
        /* transmitter mode */
        p_obj->super.tx_buff.pos = p_obj->super.tx_buff.length;
        i2c_disable_irq(&p_obj->super);
        I2C_disable_interrupt(&p_obj->super.i2c);
        if (p_obj->super.info.asynch.stop)
        {
            I2C_stop_condition(&p_obj->super.i2c);
        }
        p_obj->super.info.asynch.event = I2C_EVENT_TRANSFER_COMPLETE;
    }
    else
    {
        /* receiver mode */
        if (p_obj->super.rx_buff.pos < p_obj->super.rx_buff.length)
        {
            if (p_obj->super.rx_buff.pos == 0)
            {
               p_obj->super.rx_buff.pos = p_obj->super.rx_buff.length - 2;
            }
            p_obj->super.rx_buff.p_buffer[p_obj->super.rx_buff.pos++] = (uint8_t)I2C_read_data(&p_obj->super.i2c);
        }
        if (p_obj->super.rx_buff.pos == p_obj->super.rx_buff.length)
        {
            i2c_disable_irq(&p_obj->super);
            I2C_disable_interrupt(&p_obj->super.i2c);
            if (p_obj->super.info.asynch.stop)
            {
                I2C_stop_condition(&p_obj->super.i2c);
            }
            p_obj->super.info.asynch.event = I2C_EVENT_TRANSFER_COMPLETE;
        }
        else
        {
            I2C_set_ack(&p_obj->super.i2c, 1);
            I2C_write_data(&p_obj->super.i2c, 0);
        }
    }
}

/*--------------------------------------------------*/
/**
  * @brief     I2C Transfer handler (for slave)
  * @param     p_obj     :i2c DMA object.
  * @retval    -
  * @note      Implement as INTI2C IRQ handler.
  */
/*--------------------------------------------------*/
static void i2c_slave_irq_handler(i2c_dma_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef DEBUG */

    I2C_clear_int_status(&p_obj->super.i2c);

    if (I2C_master(&p_obj->super.i2c) || (p_obj->info.event != EVENT_DONE))
    {
        p_obj->super.info.asynch.event = I2C_EVENT_ERROR;
        return;
    }
    if (I2C_transmitter(&p_obj->super.i2c))
    {
        /* transmitter mode */
        p_obj->super.tx_buff.pos = p_obj->super.tx_buff.length;
        i2c_disable_irq(&p_obj->super);
        I2C_disable_interrupt(&p_obj->super.i2c);
        I2C_slave_init(&p_obj->super.i2c);
        p_obj->super.info.asynch.event = I2C_EVENT_TRANSFER_COMPLETE;
    }
    else
    {
        /* receiver mode */
        if (p_obj->super.rx_buff.pos < p_obj->super.rx_buff.length)
        {
            if (p_obj->super.rx_buff.pos == 0)
            {
               p_obj->super.rx_buff.pos = p_obj->super.rx_buff.length - 2;
            }
            p_obj->super.rx_buff.p_buffer[p_obj->super.rx_buff.pos++] = (uint8_t)I2C_read_data(&p_obj->super.i2c);
        }
        if (p_obj->super.rx_buff.pos == p_obj->super.rx_buff.length)
        {
            i2c_disable_irq(&p_obj->super);
            I2C_disable_interrupt(&p_obj->super.i2c);
            I2C_slave_init(&p_obj->super.i2c);
            p_obj->super.info.asynch.event = I2C_EVENT_TRANSFER_COMPLETE;
        }
        else
        {
            I2C_set_ack(&p_obj->super.i2c, 1);
            I2C_write_data(&p_obj->super.i2c, 0);
        }
    }
}

/**
 *  @}
 */ /* End of group UTILITIES_Private_functions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UTILITIES_Exported_functions
 *  @{
 */
/*--------------------------------------------------*/
/**
  * @brief     Initialize the I2C driver for master DMA
  * @param     p_obj     :i2c DMA object.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      For a Master DMA function.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result i2c_init_dma(i2c_dma_t *p_obj)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->init.dma.p_obj));
#endif /* #ifdef DEBUG */

    /*------------------------------*/
    /*  Construct Superclass        */
    /*------------------------------*/
    if (i2c_init(&p_obj->super, p_obj->init.port.sda, p_obj->init.port.scl) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }
    if ((result == TXZ_SUCCESS) && (p_obj->init.frequency != 0))
    {
        if (i2c_frequency(&p_obj->super, (int32_t)p_obj->init.frequency) != TXZ_SUCCESS)
        {
            result = TXZ_ERROR;
        }
    }
    /*------------------------------*/
    /*  Construct Subclass          */
    /*------------------------------*/
    if (result == TXZ_SUCCESS)
    {
        result = init_dma_driver(p_obj);
    }
    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     DMA reading data
  * @param     p_obj     :i2c DMA object.
  * @param     p_rx      :Buffer of read data.
  * @param     rx_length :Length of read data.
  * @param     address   :Slave address(7-bit) and last bit is 0.
  * @param     stop      :Stop to be generated after the transfer is done.
  * @retval    I2C_ERROR_NO_ERROR :Success.
  * @retval    I2C_ERROR_NO_SLAVE :Failure(No Slave Error).
  * @retval    I2C_ERROR_OTHERS   :Failure(Others Error).
  * @note      For a Master DMA function.
  * @note      Events of this function will be notified on i2c_irq_handler_dma.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
int32_t i2c_read_dma(i2c_dma_t *p_obj, uint8_t *p_rx, int32_t rx_length, int32_t address, int32_t stop)
{
    int32_t result = I2C_ERROR_NO_ERROR;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_rx));
    assert_param(check_param_address(address));
    assert_param(check_param_length(rx_length));
#endif /* #ifdef DEBUG */

    if (rx_length <= 2)
    {
        return (I2C_ERROR_PARAM);
    }
    if (p_obj->init.dma.rx != TXZ_ENABLE)
    {
        return (I2C_ERROR_OTHERS);
    }
    /*------------------------------*/
    /*  Disable Interrupt           */
    /*------------------------------*/
    i2c_disable_irq(&p_obj->super);
    I2C_disable_interrupt(&p_obj->super.i2c);

    /*-------------------------------------------*/
    /*  Send a Start Condition and Slave address */
    /*-------------------------------------------*/
    if (i2c_start(&p_obj->super) != TXZ_SUCCESS)
    {
        result = I2C_ERROR_OTHERS; /* now, not be occured */
    }
    if (i2c_byte_write(&p_obj->super, (int32_t)((uint32_t)address | 1U)) != I2C_ACK)
    {
        result = I2C_ERROR_NO_SLAVE;
    }

    /*------------------------------*/
    /*  DMA Setting                 */
    /*------------------------------*/
    if (result == I2C_ERROR_NO_ERROR)
    {
        dma_normal_cfg_t cfg;

        p_obj->info.event = EVENT_NON;
        p_obj->super.info.asynch.address = (uint32_t)address;
        p_obj->super.info.asynch.stop = (uint32_t)stop;
        p_obj->super.info.asynch.event = 0;
        p_obj->super.info.asynch.state = 0; /* no use */
        p_obj->super.rx_buff.p_buffer = p_rx;
        p_obj->super.rx_buff.length   = rx_length;
        p_obj->super.rx_buff.pos      = 0;
        p_obj->super.tx_buff.p_buffer = I2C_NULL;
        p_obj->super.tx_buff.length   = 0;
        p_obj->super.tx_buff.pos      = 0;
        cfg.source      = (uint32_t)&p_obj->super.i2c.p_instance->DBR;
        cfg.destination = (uint32_t)p_rx;
        cfg.control     = (uint32_t)(DMA_DST_INC_1BYTE   | DMA_DST_SIZE_1BYTE |
                                     DMA_SRC_INC_NON     | DMA_SRC_SIZE_1BYTE |
                                     DMA_ARBITRATION_1   | DMA_EX_TRANS_NUM(rx_length - 2) |
                                     DMA_BURST_USE       | DMA_CYCLE_MODE_BASE);
        if (dma_normal_startIt(p_obj->init.dma.p_obj, p_obj->info.rxch, DMA_BURST_USE, &cfg) != TXZ_SUCCESS)
        {
            result = I2C_ERROR_OTHERS;
        }
    }
    /*------------------------------*/
    /*  Start Data Transfer         */
    /*------------------------------*/
    if (result == I2C_ERROR_NO_ERROR)
    {
        I2C_clear_int_status(&p_obj->super.i2c);
        I2C_set_ack(&p_obj->super.i2c, 0);  /* n-2 is never last */
        I2C_enable_interrupt_dma(&p_obj->super.i2c, 0);
        if (I2C_read_data(&p_obj->super.i2c) == 0)
        {
            /* dummy read */
        }
        i2c_enable_irq(&p_obj->super);
    }
    else
    {
        if (i2c_stop(&p_obj->super) == TXZ_SUCCESS)
        {
            /* no processing */
        }
    }
    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     DMA sending data
  * @param     p_obj     :i2c DMA object.
  * @param     p_tx      :Buffer of write data.
  * @param     tx_length :Length of write data.
  * @param     address   :Slave address(7-bit) and last bit is 0.
  * @param     stop      :Stop to be generated after the transfer is done.
  * @retval    I2C_ERROR_NO_ERROR :Success.
  * @retval    I2C_ERROR_NO_SLAVE :Failure(No Slave Error).
  * @retval    I2C_ERROR_OTHERS   :Failure(Others Error).
  * @note      For a Master DMA function.
  * @note      Events of this function will be notified on i2c_irq_handler_dma.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
int32_t i2c_write_dma(i2c_dma_t *p_obj, uint8_t *p_tx, int32_t tx_length, int32_t address, int32_t stop)
{
    int32_t result = I2C_ERROR_NO_ERROR;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_tx));
    assert_param(check_param_address(address));
    assert_param(check_param_length(tx_length));
#endif /* #ifdef DEBUG */

    if (tx_length <= 1)
    {
        return (I2C_ERROR_PARAM);
    }
    if (p_obj->init.dma.tx != TXZ_ENABLE)
    {
        return (I2C_ERROR_OTHERS);
    }
    /*------------------------------*/
    /*  Disable Interrupt           */
    /*------------------------------*/
    i2c_disable_irq(&p_obj->super);
    I2C_disable_interrupt(&p_obj->super.i2c);

    /*-------------------------------------------*/
    /*  Send a Start Condition and Slave address */
    /*-------------------------------------------*/
    if (i2c_start(&p_obj->super) != TXZ_SUCCESS)
    {
        result = I2C_ERROR_OTHERS; /* now, not be occured */
    }
    if (i2c_byte_write(&p_obj->super, address) != I2C_ACK)
    {
        result = I2C_ERROR_NO_SLAVE;
    }

    /*------------------------------*/
    /*  DMA Setting                 */
    /*------------------------------*/
    if (result == I2C_ERROR_NO_ERROR)
    {
        dma_normal_cfg_t cfg;

        p_obj->info.event = EVENT_NON;
        p_obj->super.info.asynch.address = (uint32_t)address;
        p_obj->super.info.asynch.stop = (uint32_t)stop;
        p_obj->super.info.asynch.event = 0;
        p_obj->super.info.asynch.state = 0; /* no use */
        p_obj->super.rx_buff.p_buffer = I2C_NULL;
        p_obj->super.rx_buff.length   = 0;
        p_obj->super.rx_buff.pos      = 0;
        p_obj->super.tx_buff.p_buffer = p_tx;
        p_obj->super.tx_buff.length   = tx_length;
        p_obj->super.tx_buff.pos      = 0;
        cfg.source      = (uint32_t)p_tx + 1;
        cfg.destination = (uint32_t)&p_obj->super.i2c.p_instance->DBR;
        cfg.control     = (uint32_t)(DMA_DST_INC_NON     | DMA_DST_SIZE_1BYTE |
                                     DMA_SRC_INC_1BYTE   | DMA_SRC_SIZE_1BYTE |
                                     DMA_ARBITRATION_1   | DMA_EX_TRANS_NUM(tx_length - 1) |
                                     DMA_BURST_USE       | DMA_CYCLE_MODE_BASE);
        if (dma_normal_startIt(p_obj->init.dma.p_obj, p_obj->info.txch, DMA_BURST_USE, &cfg) != TXZ_SUCCESS)
        {
            result = I2C_ERROR_OTHERS;
        }
    }
    /*------------------------------*/
    /*  Start Data Transfer         */
    /*------------------------------*/
    if (result == I2C_ERROR_NO_ERROR)
    {
        I2C_clear_int_status(&p_obj->super.i2c);
        I2C_enable_interrupt_dma(&p_obj->super.i2c, 1);
        I2C_write_data(&p_obj->super.i2c, (uint32_t)p_obj->super.tx_buff.p_buffer[p_obj->super.tx_buff.pos++]);
        i2c_enable_irq(&p_obj->super);
    }
    else
    {
        if (i2c_stop(&p_obj->super) == TXZ_SUCCESS)
        {
            /* no processing */
        }
    }
    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Transfer IRQ handler
  * @param     p_obj     :i2c DMA object.
  * @retval    zero      :Transfer in progress.
  * @retval    non-zero  :Event information.
  * @note      Macro definition of return values is @ref I2C_Events.
  * @note      For a Master DMA function.
  * @attention This function should be implement as INTI2Cx_IRQHandler.
  */
/*--------------------------------------------------*/
uint32_t i2c_irq_handler_dma(i2c_dma_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef DEBUG */

    i2c_irq_handler(p_obj);

    return (p_obj->super.info.asynch.event & I2C_EVENT_ALL);
}

/*--------------------------------------------------*/
/**
  * @brief     Abort DMA Transfer
  * @param     p_obj     :i2c DMA object.
  * @retval    -
  * @note      For a Master DMA function.
  * @note      After error event occurred on i2c_irq_handler_dma, 
  * @note      call this function and clear error status. 
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void i2c_abort_dma(i2c_dma_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef DEBUG */

    discard_transmit(p_obj);
    discard_receive(p_obj);
    i2c_abort_asynch(&p_obj->super);
}

/*--------------------------------------------------*/
/**
  * @brief     Initialize the I2C driver for slave DMA
  * @param     p_obj     :i2c DMA object.
  * @param     address   :Address to be set.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      For a Slave DMA function.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result i2c_slave_init_dma(i2c_dma_t *p_obj, int32_t address)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(check_param_address(address));
#endif /* #ifdef DEBUG */
    /*------------------------------*/
    /*  Construct Superclass        */
    /*------------------------------*/
    if (i2c_init(&p_obj->super, p_obj->init.port.sda, p_obj->init.port.scl) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }
    if ((result == TXZ_SUCCESS) && (p_obj->init.frequency != 0))
    {
        if (i2c_frequency(&p_obj->super, (int32_t)p_obj->init.frequency) != TXZ_SUCCESS)
        {
            result = TXZ_ERROR;
        }
    }
    /*------------------------------*/
    /*  Construct Subclass          */
    /*------------------------------*/
    if (result == TXZ_SUCCESS)
    {
        result = init_dma_driver(p_obj);
    }
    /*------------------------------*/
    /*  Slave Mode Setting          */
    /*------------------------------*/
    if (result == TXZ_SUCCESS)
    {
        i2c_slave_mode(&p_obj->super, 1);
        i2c_slave_address(&p_obj->super, (uint32_t)address);
    }
    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     DMA reading data
  * @param     p_obj     :i2c DMA object.
  * @param     p_rx      :Buffer of read data.
  * @param     rx_length :Length of read data.
  * @retval    I2C_ERROR_NO_ERROR :Success.
  * @retval    I2C_ERROR_OTHERS   :Failure(Others Error).
  * @note      For a Slave DMA function.
  * @note      Events of this function will be notified on i2c_slave_irq_handler_dma.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
int32_t i2c_slave_read_dma(i2c_dma_t *p_obj, uint8_t *p_rx, int32_t rx_length)
{
    int32_t result = I2C_ERROR_NO_ERROR;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_rx));
    assert_param(check_param_length(rx_length));
#endif /* #ifdef DEBUG */

    if (rx_length <= 2)
    {
        return (I2C_ERROR_PARAM);
    }
    if (p_obj->init.dma.rx != TXZ_ENABLE)
    {
        return (I2C_ERROR_OTHERS);
    }
    /*------------------------------*/
    /*  Disable Interrupt           */
    /*------------------------------*/
    i2c_disable_irq(&p_obj->super);
    I2C_disable_interrupt(&p_obj->super.i2c);

    /*------------------------------*/
    /*  DMA Setting                 */
    /*------------------------------*/
    {
        dma_normal_cfg_t cfg;

        p_obj->info.event = EVENT_NON;
        p_obj->super.info.asynch.address = 0;
        p_obj->super.info.asynch.stop = 0;
        p_obj->super.info.asynch.event = 0;
        p_obj->super.info.asynch.state = 0; /* no use */
        p_obj->super.rx_buff.p_buffer = p_rx;
        p_obj->super.rx_buff.length   = rx_length;
        p_obj->super.rx_buff.pos      = 0;
        p_obj->super.tx_buff.p_buffer = I2C_NULL;
        p_obj->super.tx_buff.length   = 0;
        p_obj->super.tx_buff.pos      = 0;
        cfg.source      = (uint32_t)&p_obj->super.i2c.p_instance->DBR;
        cfg.destination = (uint32_t)p_rx;
        cfg.control     = (uint32_t)(DMA_DST_INC_1BYTE   | DMA_DST_SIZE_1BYTE |
                                     DMA_SRC_INC_NON     | DMA_SRC_SIZE_1BYTE |
                                     DMA_ARBITRATION_1   | DMA_EX_TRANS_NUM(rx_length - 2) |
                                     DMA_BURST_USE       | DMA_CYCLE_MODE_BASE);
        if (dma_normal_startIt(p_obj->init.dma.p_obj, p_obj->info.rxch, DMA_BURST_USE, &cfg) != TXZ_SUCCESS)
        {
            result  = I2C_ERROR_OTHERS;
        }
    }
    /*------------------------------*/
    /*  Start Data Transfer         */
    /*------------------------------*/
    if (result == I2C_ERROR_NO_ERROR)
    {
        I2C_clear_int_status(&p_obj->super.i2c);
        I2C_set_ack(&p_obj->super.i2c, 0);   /* n-2 is never last */
        I2C_enable_interrupt_dma(&p_obj->super.i2c, 0);
        if (I2C_read_data(&p_obj->super.i2c) == 0)
        {
            /* dummy read */
        }
        i2c_enable_irq(&p_obj->super);
    }
    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     DMA sending data
  * @param     p_obj     :i2c DMA object.
  * @param     p_tx      :Buffer of write data.
  * @param     tx_length :Length of write data.
  * @retval    I2C_ERROR_NO_ERROR :Success.
  * @retval    I2C_ERROR_OTHERS   :Failure(Others Error).
  * @note      For a Slave DMA function.
  * @note      Events of this function will be notified on i2c_slave_irq_handler_dma.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
int32_t i2c_slave_write_dma(i2c_dma_t *p_obj, uint8_t *p_tx, int32_t tx_length)
{
    int32_t result = I2C_ERROR_NO_ERROR;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_tx));
    assert_param(check_param_length(tx_length));
#endif /* #ifdef DEBUG */

    if (tx_length <= 1)
    {
        return (I2C_ERROR_PARAM);
    }
    if (p_obj->init.dma.tx != TXZ_ENABLE)
    {
        return (I2C_ERROR_OTHERS);
    }
    /*------------------------------*/
    /*  Disable Interrupt           */
    /*------------------------------*/
    i2c_disable_irq(&p_obj->super);
    I2C_disable_interrupt(&p_obj->super.i2c);

    /*------------------------------*/
    /*  DMA Setting                 */
    /*------------------------------*/
    {
        dma_normal_cfg_t cfg;

        p_obj->info.event = EVENT_NON;
        p_obj->super.info.asynch.address = 0;
        p_obj->super.info.asynch.stop = 0;
        p_obj->super.info.asynch.event = 0;
        p_obj->super.info.asynch.state = 0; /* no use */
        p_obj->super.rx_buff.p_buffer = I2C_NULL;
        p_obj->super.rx_buff.length   = 0;
        p_obj->super.rx_buff.pos      = 0;
        p_obj->super.tx_buff.p_buffer = p_tx;
        p_obj->super.tx_buff.length   = tx_length;
        p_obj->super.tx_buff.pos      = 0;
        cfg.source      = (uint32_t)p_tx + 1;
        cfg.destination = (uint32_t)&p_obj->super.i2c.p_instance->DBR;
        cfg.control     = (uint32_t)(DMA_DST_INC_NON     | DMA_DST_SIZE_1BYTE |
                                     DMA_SRC_INC_1BYTE   | DMA_SRC_SIZE_1BYTE |
                                     DMA_ARBITRATION_1   | DMA_EX_TRANS_NUM(tx_length - 1) |
                                     DMA_BURST_USE       | DMA_CYCLE_MODE_BASE);
        if (dma_normal_startIt(p_obj->init.dma.p_obj, p_obj->info.txch, DMA_BURST_USE, &cfg) != TXZ_SUCCESS)
        {
            result  = I2C_ERROR_OTHERS;
        }
    }
    /*------------------------------*/
    /*  Start Data Transfer         */
    /*------------------------------*/
    if (result == I2C_ERROR_NO_ERROR)
    {
        I2C_clear_int_status(&p_obj->super.i2c);
        I2C_enable_interrupt_dma(&p_obj->super.i2c, 1);
        I2C_write_data(&p_obj->super.i2c, (uint32_t)p_obj->super.tx_buff.p_buffer[p_obj->super.tx_buff.pos++]);
        i2c_enable_irq(&p_obj->super);
    }
    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Transfer IRQ handler
  * @param     p_obj     :i2c DMA object.
  * @retval    zero      :Transfer in progress.
  * @retval    non-zero  :Event information.
  * @note      Macro definition of return values is @ref I2C_Events.
  * @note      For a Slave DMA function.
  * @attention This function should be implement as INTI2Cx_IRQHandler.
  */
/*--------------------------------------------------*/
uint32_t i2c_slave_irq_handler_dma(i2c_dma_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef DEBUG */

    i2c_slave_irq_handler(p_obj);

    return (p_obj->super.info.asynch.event & I2C_EVENT_ALL);
}

/*--------------------------------------------------*/
/**
  * @brief     Abort Transfer
  * @param     p_obj     :i2c DMA object.
  * @retval    -
  * @note      For a Slave DMA function.
  * @note      After error event occurred on i2c_slave_irq_handler_dma, 
  * @note      call this function and clear error status. 
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void i2c_slave_abort_dma(i2c_dma_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef DEBUG */

    discard_transmit(p_obj);
    discard_receive(p_obj);
    i2c_slave_abort_asynch(&p_obj->super);
}

/** 
 *  @}
 */ /* End of group UTILITIES_Exported_functions */

/**
 *  @}
 */ /* End of group UTILITIES */

/**
 *  @} 
 */ /* End of group Example */

#endif /* defined(__I2C_API_DMA_H)  */
#ifdef __cplusplus
}
#endif /* __cplusplus */
