/**
 *******************************************************************************
 * @file    txz_uart_mdma.c
 * @brief   This file provides API functions for UART driver.
 * @brief   If MDMA transfer is used, it is necessary.
 * @version V1.1.0.0
 * $Date:: 2018-04-02 19:31:41 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_uart_mdma.h"

#if defined(__UART_MDMA_H)

/**
 *  @addtogroup Periph_Driver
 *  @{
 */

/** 
 *  @addtogroup UART
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UART_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group UART_Private_define */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UART_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group UART_Private_define */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UART_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group UART_Private_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UART_Private_typedef
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group UART_Private_typedef */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UART_Private_fuctions
 *  @{
 */
static void tx_transfer_done(uint32_t id, TXZ_Result result);
static void rx_transfer_done(uint32_t id, TXZ_Result result);
/*--------------------------------------------------*/
/**
  * @brief     MDMA TX Transfer Done Handler.
  * @param     ch     :MDMA channel.
  * @param     result :Result.
  * @retval    -
  * @note      This Function is called by DMA driver.
  */
/*--------------------------------------------------*/
static void tx_transfer_done(uint32_t id, TXZ_Result result)
{
    uart_mdma_t *p_obj = (uart_mdma_t *)id;
    if(result == TXZ_SUCCESS)
    {
        /* UART Transmit Handler Call. */
        if ((p_obj != UART_NULL) && (p_obj->super.transmit.handler != UART_NULL))
        {
            p_obj->super.transmit.handler(p_obj->super.init.id, result);
        }
    }
}

/*--------------------------------------------------*/
/**
  * @brief     MDMA RX Transfer Done Handler.
  * @param     ch     :MDMA channel.
  * @param     result :Result.
  * @retval    -
  * @note      This Function is called by DMA driver.
  */
/*--------------------------------------------------*/
static void rx_transfer_done(uint32_t id, TXZ_Result result)
{
    uart_mdma_t *p_obj = (uart_mdma_t *)id;

    if(result == TXZ_SUCCESS)
    {
        /* UART Transmit Handler Call. */
        if ((p_obj != UART_NULL) && (p_obj->super.receive.handler != UART_NULL))
        {
            p_obj->super.receive.handler(p_obj->super.init.id, result, &p_obj->super.receive.info);
        }
    }
}

/**
 *  @}
 */ /* End of group UART_Private_functions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UART_Exported_functions
 *  @{
 */
/*--------------------------------------------------*/
/**
  * @brief     Initialize the UART MDMA object.
  * @param     p_obj   :UART MDMA object.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result uart_mdma_init(uart_mdma_t *p_obj)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    /*------------------------------*/
    /*  Construct Superclass        */
    /*------------------------------*/
    if (uart_init(&p_obj->super) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }
    /*------------------------------*/
    /*  Construct Subclass          */
    /*------------------------------*/
    if (result == TXZ_SUCCESS)
    {
        __IO uint32_t cr1 = p_obj->super.p_instance->CR1;
        
        /*--- DMA Driver "Transmit Channel" ---*/
        if (p_obj->init.tx.type != UART_MDMA_TYPE_UNUSE)
        {
           TSB_UART_TypeDef *uart_reg = p_obj->super.p_instance;
           uint32_t ch = 0;
           uint32_t reg = 0;
           
            cr1 &= ~UARTxCR1_DMATE_MASK;
            cr1 |=  UARTxCR1_DMATE_ENABLE;
            
            if(uart_reg == TSB_UART0)
            {
                ch = 9;
                reg = TSB_TSEL0->CR2;
                reg &= 0xFFFF00FF;
                reg |= 0x00002300;
                TSB_TSEL0->CR2 = reg;    /* INSEL9: MDMA CH9: UART CH0 TX: UART0TX_DMAREQ */
            }
            else if(uart_reg == TSB_UART1)
            {
                ch = 13;
                reg = TSB_TSEL0->CR3;
                reg &= 0xFFFF00FF;
                reg |= 0x00002300;
                TSB_TSEL0->CR3 = reg;    /* INSEL13: MDMA CH13: UART CH1 TX: UART1TX_DMAREQ */
            }
            else if(uart_reg == TSB_UART2)
            {
                ch = 21;
                reg = TSB_TSEL0->CR5;
                reg &= 0xFFFF00FF;
                reg |= 0x00002300;
                TSB_TSEL0->CR5 = reg;    /* INSEL21: MDMA CH21: UART CH2 TX: UART2TX_DMAREQ */
            }
            else if(uart_reg == TSB_UART3)
            {
                ch = 29;
                reg = TSB_TSEL0->CR7;
                reg &= 0xFFFF00FF;
                reg |= 0x00002300;
                TSB_TSEL0->CR7 = reg;    /* INSEL29: MDMA CH29: UART CH3 TX: UART3TX_DMAREQ */
            }
            else if(uart_reg == TSB_UART4)
            {
                ch = 1;
                reg = TSB_TSEL0->CR0;
                reg &= 0xFFFF00FF;
                reg |= 0x00002300;
                TSB_TSEL0->CR0 = reg;    /* INSEL1: MDMA CH1: UART CH4 TX: UART4TX_DMAREQ */
            }
            else if(uart_reg == TSB_UART5)
            {
                ch = 31;
                reg = TSB_TSEL0->CR7;
                reg &= 0x00FFFFFF;
                reg |= 0x23000000;
                TSB_TSEL0->CR7 = reg;    /* INSEL31: MDMA CH31: UART CH5 TX: UART5TX_DMAREQ */
            }
            else
            {
            #ifndef TEST_DEBUG
                result = TXZ_ERROR;
            #else
                ch = 9;
            #endif
            }
            if(result == TXZ_SUCCESS)
            {
                mdma_ch_interrupt_setting_t int_ch;
                
                p_obj->init.tx.ch = ch;
                int_ch.priority = MDMA_PRIORITY_HIGH;
                int_ch.id = (uint32_t)p_obj;
                int_ch.handler = tx_transfer_done;
                if (mdma_ch_init(ch, int_ch) != TXZ_SUCCESS)
                {
                    result = TXZ_ERROR;
                }
            }
        }
        /*--- DMA Driver "Receive Channel" ---*/
        if (p_obj->init.rx.type != UART_MDMA_TYPE_UNUSE)
        {
            TSB_UART_TypeDef *uart_reg = p_obj->super.p_instance;
            uint32_t ch = 0;
            uint32_t reg = 0;
            
            cr1 &= ~UARTxCR1_DMARE_MASK;
            cr1 |=  UARTxCR1_DMARE_ENABLE;
            if(uart_reg == TSB_UART0)
            {
                ch = 7;
                reg = TSB_TSEL0->CR1;
                reg &= 0x00FFFFFF;
                reg |= 0x23000000;
                TSB_TSEL0->CR1 = reg;    /* INSEL7: MDMA CH7: UART CH0 RX: UART0RX_DMAREQ */
            }
            else if(uart_reg == TSB_UART1)
            {
                ch = 11;
                reg = TSB_TSEL0->CR2;
                reg &= 0x00FFFFFF;
                reg |= 0x23000000;
                TSB_TSEL0->CR2 = reg;    /* INSEL11: MDMA CH11: UART CH1 RX: UART1RX_DMAREQ */
            }
            else if(uart_reg == TSB_UART2)
            {
                ch = 19;
                reg = TSB_TSEL0->CR4;
                reg &= 0x00FFFFFF;
                reg |= 0x23000000;
                TSB_TSEL0->CR4 = reg;    /* INSEL19: MDMA CH19: UART CH2 RX: UART2RX_DMAREQ */
            }
            else if(uart_reg == TSB_UART3)
            {
                ch = 28;
                reg = TSB_TSEL0->CR7;
                reg &= 0xFFFFFF00;
                reg |= 0x00000023;
                TSB_TSEL0->CR7 = reg;    /* INSEL28: MDMA CH28: UART CH3 TX: UART3RX_DMAREQ */
            }
            else if(uart_reg == TSB_UART4)
            {
                ch = 0;
                reg = TSB_TSEL0->CR0;
                reg &= 0xFFFFFF00;
                reg |= 0x00000023;
                TSB_TSEL0->CR0 = reg;    /* INSEL1: MDMA CH0: UART CH4 RX: UART4RX_DMAREQ */
            }
            else if(uart_reg == TSB_UART5)
            {
                ch = 30;
                reg = TSB_TSEL0->CR7;
                reg &= 0xFF00FFFF;
                reg |= 0x00230000;
                TSB_TSEL0->CR7 = reg;    /* INSEL30: MDMA CH30: UART CH5 RX: UART5RX_DMAREQ */
            }
            else
            {
            #ifndef TEST_DEBUG
                result = TXZ_ERROR;
            #else
                ch = 7;
            #endif
            }
            if(result == TXZ_SUCCESS)
            {
                mdma_ch_interrupt_setting_t int_ch;
                
                p_obj->init.rx.ch = ch;
                int_ch.priority = MDMA_PRIORITY_HIGH;
                int_ch.id = (uint32_t)p_obj;
                int_ch.handler = rx_transfer_done;
                if (mdma_ch_init(ch, int_ch) != TXZ_SUCCESS)
                {
                    result = TXZ_ERROR;
                }
            }
        
        }
        if(result == TXZ_SUCCESS){
            /*--- UARTxCR1 ---*/
            p_obj->super.p_instance->CR1 = cr1;
        }
    }

    return (result);
}
/*--------------------------------------------------*/
/**
  * @brief     Release the UART MDMA object.
  * @param     -
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result uart_mdma_deinit(uart_mdma_t *p_obj)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    /*------------------------------*/
    /*  Destruct Subclass           */
    /*------------------------------*/
    /*--- DMA Driver "Transmit Channel" ---*/
    if (p_obj->init.tx.type != UART_MDMA_TYPE_UNUSE)
    {
        __IO uint32_t cr1 = p_obj->super.p_instance->CR1;
        TSB_UART_TypeDef *uart_reg = p_obj->super.p_instance;
        uint32_t ch = 0;

        /*--- UARTxCR1   ---*/
        cr1 &= ~UARTxCR1_DMATE_MASK;
        cr1 |=  UARTxCR1_DMATE_DISABLE;
        p_obj->super.p_instance->CR1 = cr1;
        /*--- DMA Driver ---*/
        if(uart_reg == TSB_UART0)
        {
            ch = 9;
            TSB_TSEL0->CR2  &= 0xFFFF00FF;              /* INSEL9: MDMA CH9: UART CH0 TX: UART0TX_DMAREQ */
        }
        else if(uart_reg == TSB_UART1)
        {
            ch = 13;
            TSB_TSEL0->CR3 &= 0xFFFF00FF;                /* INSEL13: MDMA CH13: UART CH1 TX: UART1TX_DMAREQ */
        }
        else if(uart_reg == TSB_UART2)
        {
            ch = 21;
            TSB_TSEL0->CR5 &= 0xFFFF00FF;                /* INSEL21: MDMA CH21: UART CH2 TX: UART2TX_DMAREQ */
        }
        else if(uart_reg == TSB_UART3)
        {
            ch = 29;
            TSB_TSEL0->CR7 &= 0xFFFF00FF;                 /* INSEL29: MDMA CH29: UART CH3 TX: UART3TX_DMAREQ */
        }
        else if(uart_reg == TSB_UART4)
        {
            ch = 1;
            TSB_TSEL0->CR0 &= 0xFFFF00FF;                 /* INSEL1: MDMA CH1: UART CH4 TX: UART4TX_DMAREQ */
        }
        else if(uart_reg == TSB_UART5)
        {
            ch = 31;
            TSB_TSEL0->CR7 &= 0x00FFFFFF;                 /* INSEL31: MDMA CH31: UART CH5 TX: UART5TX_DMAREQ */
        }
        else
        {
        #ifndef TEST_DEBUG
            result = TXZ_ERROR;
        #else
            ch = 9;
        #endif
        }
        if(result == TXZ_SUCCESS)
        {
            if (mdma_ch_deinit(ch) != TXZ_SUCCESS)
            {
                result = TXZ_ERROR;
            }
        }
    }
    /*--- DMA Driver "Receive Channel" ---*/
    if (p_obj->init.rx.type != UART_MDMA_TYPE_UNUSE)
    {
        __IO uint32_t cr1 = p_obj->super.p_instance->CR1;
        TSB_UART_TypeDef *uart_reg = p_obj->super.p_instance;
        uint32_t ch = 0;

        /*--- UARTxCR1   ---*/
        cr1 &= ~UARTxCR1_DMARE_MASK;
        cr1 |=  UARTxCR1_DMARE_DISABLE;
        p_obj->super.p_instance->CR1 = cr1;
        /*--- DMA Driver ---*/
        if(uart_reg == TSB_UART0)
        {
            ch = 7;
            TSB_TSEL0->CR1 &= 0x00FFFFFF;                /* INSEL7: MDMA CH7: UART CH0 RX: UART0RX_DMAREQ */
        }
        else if(uart_reg == TSB_UART1)
        {
            ch = 11;
            TSB_TSEL0->CR2 &= 0x00FFFFFF;                /* INSEL11: MDMA CH11: UART CH1 RX: UART1RX_DMAREQ */
        }
        else if(uart_reg == TSB_UART2)
        {
            ch = 19;
            TSB_TSEL0->CR4 &= 0x00FFFFFF;                /* INSEL19: MDMA CH19: UART CH2 RX: UART2RX_DMAREQ */
        }
        else if(uart_reg == TSB_UART3)
        {
            ch = 28;
            TSB_TSEL0->CR7 &= 0xFFFFFF00;                /* INSEL28: MDMA CH28: UART CH3 TX: UART3RX_DMAREQ */
        }
        else if(uart_reg == TSB_UART4)
        {
            ch = 0;
            TSB_TSEL0->CR0 &= 0xFFFFFF00;                /* INSEL1: MDMA CH0: UART CH4 RX: UART4RX_DMAREQ */
        }
        else if(uart_reg == TSB_UART5)
        {
            ch = 30;
            TSB_TSEL0->CR7 &= 0xFF00FFFF;                /* INSEL30: MDMA CH30: UART CH5 RX: UART5RX_DMAREQ */
        }
        else
        {
        #ifndef TEST_DEBUG
            result = TXZ_ERROR;
        #else
            ch = 7;
        #endif
        }
        if(result == TXZ_SUCCESS){
            if (mdma_ch_deinit(ch) != TXZ_SUCCESS)
            {
                result = TXZ_ERROR;
            }
        }
    }
    /*------------------------------*/
    /*  Destruct Superclass         */
    /*------------------------------*/
    if (uart_deinit(&p_obj->super) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }

    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Discard transmit.
  * @param     p_obj   :UART MDMA object.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      This function clears transmit's fifo, end flag and error info.
  * @attention This function is not available in interrupt.
  * @attention Use after interrupt is disabled.
  */
/*--------------------------------------------------*/
TXZ_Result uart_mdma_discard_transmit(uart_mdma_t *p_obj)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    /* Check the UART_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef DEBUG */
    /*------------------------------*/
    /*  Stop DMA                    */
    /*------------------------------*/
    if (p_obj->init.tx.type != UART_MDMA_TYPE_UNUSE)
    {
        if ( mdma_stop_ch(p_obj->init.tx.ch) != TXZ_SUCCESS)
        {
            result = TXZ_ERROR;
        }
    }
    /*------------------------------*/
    /*  Use Super Class             */
    /*------------------------------*/
    if (uart_discard_transmit(&p_obj->super) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }

    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Discard receive.
  * @param     p_obj   :UART MDMA object.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      This function clears receive's fifo, end flag and error info.
  * @attention This function is not available in interrupt.
  * @attention Use after interrupt is disabled.
  */
/*--------------------------------------------------*/
TXZ_Result uart_mdma_discard_receive(uart_mdma_t *p_obj)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    /* Check the UART_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef DEBUG */
    /*------------------------------*/
    /*  Stop DMA                    */
    /*------------------------------*/
    if (p_obj->init.rx.type != UART_MDMA_TYPE_UNUSE)
    {
        if ( mdma_stop_ch(p_obj->init.rx.ch) != TXZ_SUCCESS)
        {
            result = TXZ_ERROR;
        }
    }
    /*------------------------------*/
    /*  Use Super Class             */
    /*------------------------------*/
    if (uart_discard_receive(&p_obj->super) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }

    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief  Transmit data. Non-Blocking Communication.
  * @param  p_obj   :UART MDMA object.
  * @param  p_info  :The information of transmit data.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   Asynchronous Processing.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result uart_mdma_transmitIt(uart_mdma_t *p_obj, uart_transmit_t *p_info)
{
    TXZ_Result result  = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    /*------------------------------*/
    /*  Disable Transmit            */
    /*------------------------------*/
    /*--- UARTxTRANS ---*/
    /* Write to TXE(=0).               */
    /* Bitband Access.                 */
    (void)uart_mdma_TXE_disable(p_obj);
    /*------------------------------*/
    /*  DMA Setting                 */
    /*------------------------------*/
    {
    	mdma_descriptor_set_t desc_set_param;
    	uint32_t ch = p_obj->init.tx.ch;
    	uint32_t ttype = p_obj->init.tx.type;
    	mdma_descriptor_set_t *desc_ptr = &desc_set_param;
    	uint32_t desc_num = 0;
    	uint32_t infini = 0;
    	
    	p_obj->super.transmit.info.tx8.p_data  = p_info->tx8.p_data;
    	p_obj->super.transmit.info.tx8.num     = p_info->tx8.num;
    	if(ttype == UART_MDMA_TYPE_AREA_TO_FIX)
    	{
    		desc_ptr->dmode = MDMA_DMODE_INC;
    		desc_ptr->umode = MDMA_UMODE_TRANS_UNI;
    		desc_ptr->usize = MDMA_USIZE_1BYTE;
    		desc_ptr->ttype = MDMA_TTYPE_AREA_FIX;
    		desc_ptr->xfsad = (uint32_t)p_info->tx8.p_data;
    		desc_ptr->xfdad = (uint32_t)&p_obj->super.p_instance->DR;;
    		desc_ptr->xfsiz = p_info->tx8.num;
    		desc_ptr->dsads = (uint32_t)UART_NULL;
    		(void)mdma_desc_ch_configure(ch, desc_ptr, desc_num, infini);
    		(void)mdma_basic_start_ch(ch, MDMA_INTERRUPT_ENABLE);
    	}
    	else
    	{
    		result  = TXZ_ERROR;
    		
    	}
    
    }
    /*------------------------------*/
    /*  Enable Transmit             */
    /*------------------------------*/
    /*--- UARTxTRANS ---*/
    /* Write to TXE(=1).               */
    /* Bitband Access.                 */
    (void)uart_mdma_TXE_enable(p_obj);

    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief  Receive data. Non-Blocking Communication.
  * @param  p_obj   :UART DMA object.
  * @param  p_info  :The information of receive buffer.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   Asynchronous Processing.
  * @attention  "p_info->rx8(or rx16).num" must be over FIFO max(Refer @ref UART_FifoMax) num.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result uart_mdma_receiveIt(uart_mdma_t *p_obj, uart_receive_t *p_info)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    /*------------------------------*/
    /*  Disable Receive             */
    /*------------------------------*/
    /* Write to RXE(=0).               */
    /* Bitband Access.                 */
    (void)uart_mdma_RXE_disable(p_obj);
    /*------------------------------*/
    /*  DMA Setting                 */
    /*------------------------------*/
    {
    	mdma_descriptor_set_t desc_set_param;
    	uint32_t ch = p_obj->init.rx.ch;
    	uint32_t ttype = p_obj->init.rx.type;
    	mdma_descriptor_set_t *desc_ptr = &desc_set_param;
    	uint32_t desc_num = 0;
    	uint32_t infini = 0;
    
    	p_obj->super.receive.info.rx8.p_data  = p_info->rx8.p_data;
    	p_obj->super.receive.info.rx8.num     = p_info->rx8.num;
    	if(ttype == UART_MDMA_TYPE_FIX_TO_AREA)
    	{
    		desc_ptr->dmode = MDMA_DMODE_INC;
    		desc_ptr->umode = MDMA_UMODE_TRANS_UNI;
    		desc_ptr->usize = MDMA_USIZE_1BYTE;
    		desc_ptr->ttype = MDMA_TTYPE_FIX_AREA;
    		desc_ptr->xfsad = (uint32_t)&p_obj->super.p_instance->DR;
    		desc_ptr->xfdad = (uint32_t)p_info->rx8.p_data;
    		desc_ptr->xfsiz = p_info->rx8.num;
    		desc_ptr->dsads = (uint32_t)UART_NULL;
    		(void)mdma_desc_ch_configure(ch, desc_ptr, desc_num, infini);
    		(void)mdma_basic_start_ch(ch, MDMA_INTERRUPT_ENABLE);
    	}
    	else
    	{
    		result  = TXZ_ERROR;
    		
    	}
    }
    /*------------------------------*/
    /*  Enable Receive              */
    /*------------------------------*/
    /*--- UARTxTRANS ---*/
    /* Write to RXE(=1).               */
    /* Bitband Access.                 */
    (void)uart_mdma_RXE_enable(p_obj);

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Uart ch to MDMA ch
  * @param  uart_ch :Uart channel.
  * @param  tx_ch :MDMA uart tx channel.
  * @param  rx_ch :MDMA uart rx channel.
  * @return -
  * @retval -
  * @note   -
  * @attention -
  */
/*--------------------------------------------------*/
TXZ_Result uart_mdma_get_uart_ch_to_mdma_ch(uint32_t uart_ch, uint32_t *tx_ch, uint32_t *rx_ch)
{
    TXZ_Result result = TXZ_SUCCESS;
    
    switch(uart_ch)
    {
    case UART_MDMA_UART_CH0:
        *tx_ch = 9;
        *rx_ch = 7;
        break;
    case UART_MDMA_UART_CH1:
        *tx_ch = 13;
        *rx_ch = 11;
        break;
    case UART_MDMA_UART_CH2:
        *tx_ch = 21;
        *rx_ch = 19;
        break;
    case UART_MDMA_UART_CH3:
        *tx_ch = 29;
        *rx_ch = 28;
        break;
    case UART_MDMA_UART_CH4:
        *tx_ch = 1;
        *rx_ch = 0;
        break;
    case UART_MDMA_UART_CH5:
        *tx_ch = 31;
        *rx_ch = 30;
        break;
    default:
        result = TXZ_ERROR;
        break;
    }
    return (result);
}

TXZ_Result uart_mdma_TXE_disable(uart_mdma_t *p_obj)
{
    TXZ_Result result  = TXZ_SUCCESS;

    __IO uint32_t cr1 = p_obj->super.p_instance->CR1;
    
    cr1 &= ~UARTxCR1_DMATE_MASK;
    cr1 |=  UARTxCR1_DMATE_DISABLE;
    p_obj->super.p_instance->CR1 = cr1;
    /*------------------------------*/
    /*  Disable Transmit            */
    /*------------------------------*/
    /*--- UARTxTRANS ---*/
    /* Write to TXE(=0).               */
    /* Bitband Access.                 */
    disable_UARTxTRANS_TXE(p_obj->super.p_instance);
    return (result);
}
TXZ_Result uart_mdma_TXE_enable(uart_mdma_t *p_obj)
{
    TXZ_Result result  = TXZ_SUCCESS;

    __IO uint32_t cr1 = p_obj->super.p_instance->CR1;

    cr1 &= ~UARTxCR1_DMATE_MASK;
    cr1 |=  UARTxCR1_DMATE_ENABLE;
    p_obj->super.p_instance->CR1 = cr1;
    /*------------------------------*/
    /*  Enable Transmit            */
    /*------------------------------*/
    /*--- UARTxTRANS ---*/
    /* Write to TXE(=0).               */
    /* Bitband Access.                 */
    enable_UARTxTRANS_TXE(p_obj->super.p_instance);
    return (result);
}
TXZ_Result uart_mdma_RXE_disable(uart_mdma_t *p_obj)
{
    TXZ_Result result  = TXZ_SUCCESS;
    __IO uint32_t cr1 = p_obj->super.p_instance->CR1;
     
    cr1 &= ~UARTxCR1_DMARE_MASK;
    cr1 |=  UARTxCR1_DMARE_DISABLE;
    p_obj->super.p_instance->CR1 = cr1;

    /*------------------------------*/
    /*  Disable Transmit            */
    /*------------------------------*/
    /*--- UARTxTRANS ---*/
    /* Write to RXE(=0).               */
    /* Bitband Access.                 */
    disable_UARTxTRANS_RXE(p_obj->super.p_instance);
    return (result);
}
TXZ_Result uart_mdma_RXE_enable(uart_mdma_t *p_obj)
{
    TXZ_Result result  = TXZ_SUCCESS;
    __IO uint32_t cr1 = p_obj->super.p_instance->CR1;

    cr1 &= ~UARTxCR1_DMARE_MASK;
    cr1 |=  UARTxCR1_DMARE_ENABLE;
    p_obj->super.p_instance->CR1 = cr1;
    /*------------------------------*/
    /*  Enable Transmit            */
    /*------------------------------*/
    /*--- UARTxTRANS ---*/
    /* Write to RXE(=1).               */
    /* Bitband Access.                 */
    enable_UARTxTRANS_RXE(p_obj->super.p_instance);
    return (result);
}

/**
 *  @}
 */ /* End of group UART_Exported_functions */

/**
 *  @} 
 */ /* End of group UART */

/**
 *  @} 
 */ /* End of group Periph_Driver */

#endif /* defined(__UART_MDMA_H)  */

#ifdef __cplusplus
}
#endif /* __cplusplus */

