/** 
 *******************************************************************************
 * @file    main.c
 * @brief   HDMA HDMA_MEM_MEM Sample Application.
 * @version V1.0.0.2
 * $Date:: 2018-04-02 19:31:41 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
   
#include "txz_sample_def.h"
#include "bsp.h"
#include "bsp_uart_io.h"
#include "txz_tspi_hdma.h"
/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup HDMA_MEM_MEM HDMA Memory to Memory Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup HDMA_MEM_MEM_Private_define HDMA Memory to Memory Private Define
 *  @{
 */

/*----------------------*/
/*  DMA Setting         */
/*----------------------*/
#define CFG_DATA_SIZE         ((uint32_t)127)            /* DMA assigned data size  */
#define CFG_DMA_CH            ((uint32_t)0)              /* DMA Using Channel       */
/** 
 *  @}
 */ /* End of group HDMA_MEM_MEM_Private_define */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup HDMA_MEM_MEM_Private_define HDMA Memory to Memory Private Define
 *  @{
 */

#define MAIN_NULL               ((void *)0) /*!< Null Pointer */

/** 
 *  @}
 */ /* End of group HDMA_MEM_MEM_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup HDMA_MEM_MEM_Private_define HDMA Memory to Memory Private Define
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief  Processing mode.
*/
/*----------------------------------*/
typedef enum
{
    PROC_IDLE,              /*!< Idle.                        */
    PROC_START,             /*!< Start.                       */
    PROC_STOP,              /*!< Stop.                        */
    PROC_OUTPUT_MSG,        /*!< Output first message.        */
    PROC_INPUT,             /*!< Wait to input.               */
    PROC_RUNNING,           /*!< DMA Running.                 */
    PROC_WAIT,              /*!< DMA Transfer Complete Wait.  */
    PROC_OUTPUT_ECHO,       /*!< Output Echo back data.       */
    PROC_OUTPUT_ERR,        /*!< Output input error message.  */
    PROC_END,               /*!< End.                         */
    PROC_ERROR              /*!< Error.                       */
} Processing;

typedef enum
{
    TRANS_STATE_IDLE,       /* IDLE      */
    TRANS_STATE_RUNNING,    /* Running   */
    TRANS_STATE_SUCCESS,    /* Success   */
    TRANS_STATE_FAILURE     /* Failure   */
} TransState;

/** 
 *  @}
 */ /* End of group HDMA_MEM_MEM_Private_define */



/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup HDMA_MEM_MEM_Private_typedef HDMA Memory to Memory Private Typedef
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief Instance Information.
*/
/*----------------------------------*/
typedef struct
{
   hdma_t        hdma;
   tspi_dma_t    tspi_dma_obj;     /*!< Application :TSPI. */
   uart_io_t     *p_uart;         /*!< Application :USB UART.          */
} instance_t;

/*----------------------------------*/
/** 
 * @brief  Input data information.
*/
/*----------------------------------*/
typedef struct
{
    char    data[(UART_IO_RECEIVE_MAX+1)];  /*!< Input data(+ for NULL 1byte). */
    uint8_t wp;                                 /*!< Write pointer.                */
} input_t;

/**
 *  @}
 */ /* End of group HDMA_MEM_MEM_Private_typedef */


/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup HDMA_MEM_MEM_Private_variables HDMA Memory to Memory Private Variables
 *  @{
 */
static instance_t instance;    /*!< Instance.                  */
static Processing proc;        /*!< Processing mode.           */
static input_t    input;       /*!< Input data information.    */
static TransState status;      /*!< DMA Status.                */
static uint32_t transsize;     /*!< DMA Transfer size.         */
static tspi_transmit_t t_info; /*!< TSPI Transfer information. */
static tspi_receive_t  r_info; /*!< TSPI Receive  information. */

#if defined ( __CC_ARM   )      /* RealView Compiler */
    static __align(4) uint8_t src[CFG_DATA_SIZE];
    static __align(4) uint8_t dst[CFG_DATA_SIZE];
#elif defined ( __ICCARM__ )    /*IAR Compiler */
    #pragma location="dma_work"
    #pragma data_alignment=4
    static uint8_t src[CFG_DATA_SIZE];
    static uint8_t dst[CFG_DATA_SIZE];
#endif

/**
 *  @}
 */ /* End of group HDMA_MEM_MEM_Private_variables */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup HDMA_MEM_MEM_Private_fuctions HDMA Memory to Memory Private Fuctions
 *  @{
 */
static void transmit_handler(uint32_t id, TXZ_Result result);
static void variable_initialize(void);
static TXZ_Result driver_hdma_initialize(void);
static TXZ_Result driver_hdma_finalize(void);
static TXZ_Result application_initialize(void);
static TXZ_Result application_finalize(void);
static void clear_data(void);

/*--------------------------------------------------*/
/** 
  * @brief      Transmit handler.
  * @param[in]  p_obj   :Object address.
  * @param[in]  result  :Result.
  * @retval     -
  * @note       Called by tspi driver
  */
/*--------------------------------------------------*/
static void transmit_handler(uint32_t id, TXZ_Result result)
{
    if (result == TXZ_SUCCESS)
    {
        /*------------------------------*/
        /*  Result Check                */
        /*------------------------------*/
        if (result == TXZ_SUCCESS)
        {
            status = TRANS_STATE_SUCCESS;
        }
        else
        {
            status = TRANS_STATE_FAILURE;
        }
    }
}
/*--------------------------------------------------*/
/** 
  * @brief  Variable Initialize
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void variable_initialize(void)
{
    /* Instance. */
    instance.p_uart                              = MAIN_NULL;
    instance.tspi_dma_obj.super.p_instance       = MAIN_NULL;
    instance.hdma.p_instance                     = MAIN_NULL;
    /* Processing mode. */
    proc = PROC_START;
    /* Input data information. */
    clear_data();
}

/*--------------------------------------------------*/
/** 
  * @brief  DMA Driver Initialize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result driver_hdma_initialize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*--- Construct (Register Allocation) ---*/
    instance.hdma.p_instance = TSB_DMACA;
    instance.tspi_dma_obj.init.p_obj = &instance.hdma;
    if (hdma_init(instance.tspi_dma_obj.init.p_obj) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }

    return (result);
}



/*--------------------------------------------------*/
/** 
  * @brief  DMA Driver Finalize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result driver_hdma_finalize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*--- Finalize ---*/
    if (hdma_deinit(instance.tspi_dma_obj.init.p_obj) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }
    /*--- Destruct (Register Release) ---*/
    /* Register Allocation */
    instance.tspi_dma_obj.init.p_obj->p_instance = MAIN_NULL;

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Application Initialize
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_initialize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* Usb Uart             */
    /*----------------------*/
    /* Tx & Rx are same channel */
    {
       uart_io_initial_setting_t param;

        param.id         = 0;   /* User ID :any value. */
        switch (bsp_get_usb_uart_tx_ch(BSP_USB_UART_0))
        {
        case 0:
            param.p_reg  = TSB_UART0;
            break;
        case 1:
            param.p_reg  = TSB_UART1;
            break;
        case 2:
            param.p_reg  = TSB_UART2;
            break;
        default:
            param.p_reg  = MAIN_NULL;
            break;
        }
        param.boudrate   = bsp_get_usb_uart_tx_baudrate(BSP_USB_UART_0);
        param.irq.tx     = bsp_get_usb_uart_tx_nvic(BSP_USB_UART_0);
        param.irq.rx     = bsp_get_usb_uart_rx_nvic(BSP_USB_UART_0);
        param.irq.err    = bsp_get_usb_uart_err_nvic(BSP_USB_UART_0);
        if (param.p_reg != MAIN_NULL)
        {
            result  = uart_io_initialize(&param);
        }
        else
        {
            result = TXZ_ERROR;
        }
    }
    /*----------------------*/
    /* TSPI             */
    /*----------------------*/
    /* Tx TSPI channel. */
    {
        instance.tspi_dma_obj.super.init.id  =  1;   /* User ID :any value. */
        switch (bsp_get_tspi_tx_ch(BSP_SFLASH_2))
        {
        case 0:
            instance.tspi_dma_obj.super.p_instance  = TSB_TSPI0;
            break;
        case 1:
            instance.tspi_dma_obj.super.p_instance  = TSB_TSPI1;
            break;
        case 2:
            instance.tspi_dma_obj.super.p_instance  = TSB_TSPI2;
            break;
        case 3:
            instance.tspi_dma_obj.super.p_instance  = TSB_TSPI3;
            break;
        default:
            instance.tspi_dma_obj.super.p_instance  = MAIN_NULL;
            break;
        }
        /* Set Initial parameter of TSPIxCR1 */
        instance.tspi_dma_obj.super.init.cnt1.trxe = TSPI_DISABLE;
        instance.tspi_dma_obj.super.init.cnt1.tspims = TSPI_SPI_MODE;
        instance.tspi_dma_obj.super.init.cnt1.mstr = TSPI_MASTER_OPERATION;
        instance.tspi_dma_obj.super.init.cnt1.tmmd = TSPI_TWO_WAY;
        instance.tspi_dma_obj.super.init.cnt1.cssel = TSPI_TSPIxCS0_ENABLE;
        instance.tspi_dma_obj.super.init.cnt1.fc = TSPI_TRANS_RANGE_CONTINUE;
        /* Set Initial parameter of TSPIxCR2 */
        instance.tspi_dma_obj.super.init.cnt2.tidle = TSPI_TIDLE_HI;
        instance.tspi_dma_obj.super.init.cnt2.txdemp = TSPI_TXDEMP_HI;
        instance.tspi_dma_obj.super.init.cnt2.rxdly = TSPI_RXDLY_SET;
        instance.tspi_dma_obj.super.init.cnt2.til = TSPI_TX_FILL_LEVEL_0;
        instance.tspi_dma_obj.super.init.cnt2.ril = TSPI_RX_FILL_LEVEL_7;
        instance.tspi_dma_obj.super.init.cnt2.inttxwe = TSPI_TX_INT_DISABLE;
        instance.tspi_dma_obj.super.init.cnt2.intrxwe = TSPI_RX_INT_DISABLE;
        instance.tspi_dma_obj.super.init.cnt2.inttxfe = TSPI_TX_FIFO_INT_DISABLE;
        instance.tspi_dma_obj.super.init.cnt2.intrxfe = TSPI_RX_FIFO_INT_DISABLE;
        instance.tspi_dma_obj.super.init.cnt2.interr = TSPI_ERR_INT_DISABLE;
        instance.tspi_dma_obj.super.init.cnt2.dmate = TSPI_TX_DMA_INT_DISABLE;
        instance.tspi_dma_obj.super.init.cnt2.dmare = TSPI_RX_DMA_INT_DISABLE;
        /* Set Initial parameter of TSPIxBR */
        instance.tspi_dma_obj.super.init.brd.brck = TSPI_BR_CLOCK_1;
        instance.tspi_dma_obj.super.init.brd.brs = TSPI_BR_DIVIDER_2;
        /* Set Initial parameter of TSPIxFMTR0 */
        instance.tspi_dma_obj.super.init.fmr0.dir = TSPI_DATA_DIRECTION_MSB;
        instance.tspi_dma_obj.super.init.fmr0.fl = TSPI_DATA_LENGTH_8;
        instance.tspi_dma_obj.super.init.fmr0.fint = TSPI_INTERVAL_TIME_0;
        instance.tspi_dma_obj.super.init.fmr0.cs3pol = TSPI_TSPIxCS3_NEGATIVE;
        instance.tspi_dma_obj.super.init.fmr0.cs2pol = TSPI_TSPIxCS2_NEGATIVE;
        instance.tspi_dma_obj.super.init.fmr0.cs1pol = TSPI_TSPIxCS1_NEGATIVE;
        instance.tspi_dma_obj.super.init.fmr0.cs0pol = TSPI_TSPIxCS0_NEGATIVE;
        instance.tspi_dma_obj.super.init.fmr0.ckpha = TSPI_SERIAL_CK_2ND_EDGE;
        instance.tspi_dma_obj.super.init.fmr0.ckpol = TSPI_SERIAL_CK_IDLE_HI;
        instance.tspi_dma_obj.super.init.fmr0.csint = TSPI_MIN_IDLE_TIME_1;
        instance.tspi_dma_obj.super.init.fmr0.cssckdl = TSPI_SERIAL_CK_DELAY_1;
        instance.tspi_dma_obj.super.init.fmr0.sckcsdl = TSPI_NEGATE_1;
        /* Set Initial parameter of TSPIxFMTR1 */
        instance.tspi_dma_obj.super.init.fmr1.vpe = TSPI_PARITY_DISABLE;
        instance.tspi_dma_obj.super.init.fmr1.vpm = TSPI_PARITY_BIT_ODD;
        instance.tspi_dma_obj.super.transmit.handler = transmit_handler;
        if ( instance.tspi_dma_obj.super.p_instance != MAIN_NULL)
        {
            if (tspi_init(&instance.tspi_dma_obj.super) != TXZ_SUCCESS)
            {
                result = TXZ_ERROR;
            }
        }
        else
        {
            result = TXZ_ERROR;
        }
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Application Finalize
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_finalize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* USB UART             */
    /*----------------------*/
    if (instance.p_uart != MAIN_NULL)
    {
        uart_io_finalize();
    }
    
    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Clear input data.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void clear_data(void)
{
    uint8_t i;

    for (i=0; i<(UART_IO_RECEIVE_MAX+1); i++)
    {
        input.data[i] = 0;
    }
    input.wp = 0;
    for (i=0; i < CFG_DATA_SIZE; i++)
    {
        src[i] = 0;
    }
    for (i=0; i < CFG_DATA_SIZE; i++)
    {
        dst[i] = 0;
    }
}
/**
 *  @}
 */ /* End of group HDMA_MEM_MEM_Private_fuctions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup HDMA_MEM_MEM_Exported_functions HDMA Memory to Memory Exported Functions
 *  @{
 */
void dmac_done(uint32_t ch);
void dmac_error(void);
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    /*----------------------*/
    /* BSP initialize       */
    /*----------------------*/
    bsp_initialize();
    /*----------------------*/
    /* Initialize           */
    /*----------------------*/
    /*--- variable    ---*/
    variable_initialize();
    /*--- driver ---*/
    if (driver_hdma_initialize() != TXZ_SUCCESS)
    {
        proc = PROC_ERROR;
    }
    /*--- application ---*/
    if (application_initialize() != TXZ_SUCCESS)
    {
        proc = PROC_ERROR;
    }
    /*----------------------*/
    /* Main Processing      */
    /*----------------------*/
    while(proc != PROC_ERROR)
    {
        switch(proc)
        {
        /*--- Start ---*/
        case PROC_START:
            {
                /* DMA  */
                status = TRANS_STATE_IDLE;
                NVIC_EnableIRQ(INTHDMAATC_IRQn);
                NVIC_EnableIRQ(INTHDMAAERR_IRQn);
                proc = PROC_OUTPUT_MSG;
            }
            break;
        /*--- Stop ---*/
        case PROC_STOP:
            {
                /* DMA  */
                NVIC_DisableIRQ(INTHDMAATC_IRQn);
                NVIC_DisableIRQ(INTHDMAAERR_IRQn);
                proc = PROC_ERROR;
            }
            break;
        /*--- Output "First Message" ---*/
        case PROC_OUTPUT_MSG:
            {
                /* Clear data.     */
                clear_data();
                /* Output first message. */
                if (printf("Input = ") != -1 )
                {
                    proc = PROC_INPUT;
                }
                else
                {
                    proc = PROC_STOP;
                }
            }
            break;
        /*--- Wait to Input ---*/
        case PROC_INPUT:
            {
                int data;

                data = getchar();
                if ((data != -1) && (input.wp < 10))
                {
                    input.data[input.wp] = (char)data;
                    input.wp++;
                    switch ((char)data)
                    {
                    /*--- CR: carriage return ---*/
                    case '\r':
                        /* no process */
                        break;
                    /*--- LF: line feed ---*/
                    case '\n':
                        proc = PROC_RUNNING;
                        break;
                    /*--- default ---*/
                    default:
                        /* no process */
                        break;
                    }
                }
                else
                {
                    proc = PROC_OUTPUT_ERR;
                }
            }
            break;
        /*--- DMA Start ---*/
        case PROC_RUNNING:
            {
                uint32_t i = 0;
                transsize = input.wp; 
                for(i = 0; i < transsize ; i++){
                   src[i] = (uint8_t)input.data[i];
                }
                status = TRANS_STATE_RUNNING;
                {
                    /* DMA Driver Channel Setting */
                    instance.tspi_dma_obj.init.tx.type = TSPI_HDMA_TYPE_TX_USE;
                    instance.tspi_dma_obj.init.tx.ch   = CFG_DMA_CH;
                    instance.hdma.ch[instance.tspi_dma_obj.init.tx.ch].cfg.lock = HDMA_CH_LOCK_DISABLE;
                    instance.hdma.ch[instance.tspi_dma_obj.init.tx.ch].cfg.itc = HDMA_CH_ITC_ENABLE;
                    instance.hdma.ch[instance.tspi_dma_obj.init.tx.ch].cfg.ie = HDMA_CH_IE_ENABLE;
                    if (tspi_dma_init(&instance.tspi_dma_obj) != TXZ_SUCCESS)
                    {
                      proc = PROC_ERROR;
                    }else{
                        /* Send Data Set */
                        t_info.tx8.p_data = &src[0];
                        t_info.tx8.num = transsize;
                        /* Transfer Start */
                        if (tspi_dma_transmitIt(&instance.tspi_dma_obj, &t_info) != TXZ_SUCCESS)
                        {
                            proc = PROC_ERROR;
                        }else{
                            proc = PROC_WAIT;
                        }
                    }
                }
                break;
            }
        /*--- Wait DMA Transfer & Output Transfer data ---*/
        case PROC_WAIT:
            {
                if (status == TRANS_STATE_SUCCESS)
                {
                    status = TRANS_STATE_IDLE;
                    /* Receive Data Set */
                    r_info.rx8.p_data = &dst[0];
                    r_info.rx8.num = transsize;
                    /* Receive Start */
                    if (tspi_master_read(&instance.tspi_dma_obj.super, &r_info, 5000) != TXZ_SUCCESS)
                    {
                        proc = PROC_ERROR;
                    }else{
                        if (src[0] != dst[0]){
                            proc = PROC_ERROR;
                        }else{
                            if(printf("transdata = %s\n", (const char*)&dst[0]) != -1)
                            {
                                proc = PROC_OUTPUT_MSG;
                            }
                            else
                            {
                                proc = PROC_STOP;
                            }
                        }
                    }
                }
            }
            break;
        /*--- Output "Input Error Message" ---*/
        case PROC_OUTPUT_ERR:
            {
                if (printf("\n Input Error !! \n") != -1 )
                {
                    proc = PROC_OUTPUT_MSG;
                }
                else
                {
                    proc = PROC_STOP;
                }
            }
            break;
        /*--- Error ---*/
        case PROC_ERROR:
            /* no processing */
            break;
        /*--- Default ---*/
        default:
            proc = PROC_ERROR;
            break;
        }
    }
    /*>>> System Error. Please Debug. */
    /*----------------------*/
    /* Finalize             */
    /*----------------------*/
    /*--- application ---*/
    if (application_finalize() != TXZ_SUCCESS)
    {
        /* no processing */
    }
    /*--- driver      ---*/
    if (driver_hdma_finalize() != TXZ_SUCCESS)
    {
        /* no processing */
    }
    /*----------------------*/
    /* BSP finalize         */
    /*----------------------*/
    bsp_finalize();
#ifdef DEBUG
    assert_failed(MAIN_NULL, 0);
#endif

    return (0);
}
/*--------------------------------------------------*/
/** 
  * @brief  USB UART reception IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_rx(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_rx_irq_handler();
        break;
    /*--- usb uart 1 ---*/
    case BSP_USB_UART_1:
        uart_io_rx_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }

}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART transmit IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_tx(BSPUsbUart uart)
{
    switch (uart)
    {
        /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_tx_irq_handler();
        break;
        /*--- usb uart 1 ---*/
    case BSP_USB_UART_1:
        uart_io_tx_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART error IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_err(BSPUsbUart uart)
{
    switch (uart)
    {
        /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_err_irq_handler();
        break;
        /*--- usb uart 1 ---*/
    case BSP_USB_UART_1:
        uart_io_err_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  SPI flash transmit IRQ.
  * @param  sflash :SPI Flash.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_sflash_tx(BSPSFlash sflash)
{
    tspi_irq_handler_transmit(&instance.tspi_dma_obj.super);
}

/*--------------------------------------------------*/
/** 
  * @brief  SPI flash error IRQ.
  * @param  sflash :SPI Flash.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_sflash_err(BSPSFlash sflash)
{
    switch(sflash){
        case BSP_SFLASH_0:
            tspi_error_irq_handler(&instance.tspi_dma_obj.super);
            break;
        case BSP_SFLASH_1:
            tspi_error_irq_handler(&instance.tspi_dma_obj.super);
            break;
        case BSP_SFLASH_2:
            tspi_error_irq_handler(&instance.tspi_dma_obj.super);
            break;
        case BSP_SFLASH_3:
            tspi_error_irq_handler(&instance.tspi_dma_obj.super);
            break;
        default:
            break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  HDMAC Transfer Complete IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_dmac_done(void)
{
    if(hdma_irq_handler(instance.tspi_dma_obj.init.p_obj) != TXZ_SUCCESS)
    {
        /* no process */
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  HDMAC ERROR IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_dmac_err(void)
{
    if(hdma_error_irq_handler(instance.tspi_dma_obj.init.p_obj) != TXZ_SUCCESS)
    {
        /* no process */
    }
}

#ifdef DEBUG
/*--------------------------------------------------*/
/** 
  * @brief  Failure Assert.
  * @note   for debug
  */
/*--------------------------------------------------*/
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
/**
 *  @}
 */ /* End of group HDMA_MEM_MEM_Exported_functions */

/**
 *  @}
 */ /* End of group HDMA_MEM_MEM */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
