/**
 *******************************************************************************
 * @file    i2c_eeprom.c
 * @brief   This file provides API functions for I2C EEPROM(EEPROM-24FC256).
 * @version V1.0.0.0
 * $Date:: 2016-11-08 00:00:00 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "i2c_eeprom.h"
#include "bsp_i2c.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @addtogroup I2C_EEPROM
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup I2C_EEPROM_Private_macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group I2C_EEPROM_Private_macro */
/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup I2C_EEPROM_Private_define
 *  @{
 */

/** 
 *  @}
 */ /* End of group I2C_EEPROM_Private_define */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup I2C_EEPROM_Private_define
 *  @{
 */
#define I2C_EEPROM_ADDRESS	  ((uint8_t)0xA0) 			/*!< Slave Address.    */
#define I2C_EEPROM_CLOCK	  (400000)					/*!< Frequency.        */
/** 
 *  @}
 */ /* End of group I2C_EEPROM_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup I2C_EEPROM_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group I2C_EEPROM_Private_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup I2C_EEPROM_Private_typedef
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group I2C_EEPROM_Private_typedef */


/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup I2C_EEPROM_Private_variables
 *  @{
 */
static uint8_t wbuff[64+2];      /*!< The buffer for write data.  */
static uint8_t rbuff[64+2];      /*!< The buffer for read data.   */
static i2c_t   obj;              /*!< I2C driver instance.        */

/**
 *  @}
 */ /* End of group I2C_EEPROM_Private_variables */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup I2C_EEPROM_Private_functions
 *  @{
 */
/* no define */
/**
 *  @}
 */ /* End of group I2C_EEPROM_Private_functions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup I2C_EEPROM_Exported_functions
 *  @{
 */
/*--------------------------------------------------*/
/**
  * @brief     Initializing of I2C Driver
  * @param     -
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  */
/*--------------------------------------------------*/
TXZ_Result EEPROM_i2c_init(void)
{
    TXZ_Result result = TXZ_ERROR;

    /*------------------------------*/
    /*  I2C Driver Initialize       */
    /*------------------------------*/
	if (i2c_init(&obj, I2C_CFG_PORT_SDA, I2C_CFG_PORT_SCL) == TXZ_SUCCESS)
	{
#ifdef I2C_EEPROM_CLOCK
		if (i2c_frequency(&obj, I2C_EEPROM_CLOCK) == TXZ_SUCCESS)
		{
			result = TXZ_SUCCESS;
		}
#endif		
	}
    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     EEPROM BYTE READ
  * @param     address     :Access location Address.
  * @param     p_data      :Destination address of Read data.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  */
/*--------------------------------------------------*/
TXZ_Result EEPROM_i2c_ReadByte(int32_t address, uint8_t *p_data)
{
 	TXZ_Result result = TXZ_SUCCESS;

	/* Start */
	if (i2c_start(&obj) != TXZ_SUCCESS)
	{
		result = TXZ_ERROR;
	}
	/* Control Byte */
	if (i2c_byte_write(&obj, (int32_t)I2C_EEPROM_ADDRESS) != I2C_ACK)
	{
		result = TXZ_ERROR;
	}
	/* Adddress High Byte */
	if (i2c_byte_write(&obj, (address >> 8)) != I2C_ACK)
	{
		result = TXZ_ERROR;
	}
	/* Adddress Low Byte */
	if (i2c_byte_write(&obj, (address)) != I2C_ACK)
	{
		result = TXZ_ERROR;
	}
	/* Start */
	if (i2c_start(&obj) != TXZ_SUCCESS)
	{
		result = TXZ_ERROR;
	}
	/* Control Byte */
	if (i2c_byte_write(&obj, (int32_t)(I2C_EEPROM_ADDRESS|1)) != I2C_ACK)
	{
		result = TXZ_ERROR;
	}
	/* Data Byte */
	if (result != TXZ_ERROR)
	{
	   *p_data = (uint8_t)i2c_byte_read(&obj, 1);
	}
	/* Stop */
	if (i2c_stop(&obj) != TXZ_SUCCESS)
	{
		result = TXZ_ERROR;
	}
 	return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     EEPROM BYTE WRITE
  * @param     address     :Access location Address.
  * @param     data        :Write data.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  */
/*--------------------------------------------------*/
TXZ_Result EEPROM_i2c_WriteByte(int32_t address, uint8_t data)
{
 	TXZ_Result result = TXZ_SUCCESS;

	/* Start */
	if (i2c_start(&obj) != TXZ_SUCCESS)
	{
		result = TXZ_ERROR;
	}
	/* Control Byte */
	if (i2c_byte_write(&obj, (int32_t)I2C_EEPROM_ADDRESS) != I2C_ACK)
	{
		result = TXZ_ERROR;		
	}
	/* Adddress High Byte */
	if (i2c_byte_write(&obj, (address >> 8)) != I2C_ACK)
	{
		result = TXZ_ERROR;
	}
	/* Adddress Low Byte */
	if (i2c_byte_write(&obj, (address)) != I2C_ACK)
	{
		result = TXZ_ERROR;
	}
	/* Data Byte */
	if (i2c_byte_write(&obj, (int32_t)data) != I2C_ACK)
	{
		result = TXZ_ERROR;
	}
	/* Stop */
	if (i2c_stop(&obj) != TXZ_SUCCESS)
	{
		result = TXZ_ERROR;
	}
 	return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     EEPROM SEQUENCIAL READ
  * @param     address     :Access location Address.
  * @param     p_data      :Destination address of Read data.
  * @param     length      :Number of read bytes.
  * @retval    Number of read bytes.
  */
/*--------------------------------------------------*/
int32_t EEPROM_i2c_ReadData(int32_t address, uint8_t *p_data, int32_t length)
{
    int32_t result = -1;
	int32_t i;

	/* for the time being */
	wbuff[0] = (uint8_t)(address >> 8);
	wbuff[1] = (uint8_t)(address);

	/* EEPROM address transmit */
	if (i2c_write(&obj, (int32_t)I2C_EEPROM_ADDRESS, wbuff, 2, 0) == 2)
	{
		/* EEPROM data read */
		int32_t rx_length = 0;
			
		rx_length = i2c_read(&obj, (int32_t)I2C_EEPROM_ADDRESS, rbuff, length, 1);
		if(rx_length < length)
		{
			/* error */
		}
		else
		{
			uint8_t *p_src = &rbuff[0];
			uint8_t *p_dst = p_data;

			for (i = 0; ((i < 64) && (i < rx_length)); i++)
			{
				*p_dst++ = *p_src++;
			}
			result = i;
		}
	}
	return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     EEPROM PAGE WRITE
  * @param     address     :Access location Address.
  * @param     p_data   :Source Address of Write data.
  * @param     length   :Number of write bytes.
  * @retval    Number of write bytes.
  */
/*--------------------------------------------------*/
int32_t EEPROM_i2c_WritePage(int32_t address, uint8_t *p_data, int32_t length)
{
    int32_t result = -1;
	int32_t i;
	
	wbuff[0] = (uint8_t)(address >> 8);
	wbuff[1] = (uint8_t)(address);
	{
		uint8_t *p_src = p_data;
		uint8_t *p_dst = &wbuff[2];
		for (i = 0; ((i < 64) && (i < length)); i++)
		{
			*p_dst++ = *p_src++;
		}		
		if (i != 0)
		{
			int32_t tx_length = 0;
			tx_length = i2c_write(&obj, (int32_t)I2C_EEPROM_ADDRESS, wbuff, i+2, 1);
			if (tx_length > 2)
			{
				result = tx_length;
			}
		}
	}
    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief      EEPROM_i2c_WriteProtection
  * @param      -
  * @retval     SUCCESS :Success.
  * @retval     ERROR   :Failure.
  */
/*--------------------------------------------------*/
TXZ_Result EEPROM_i2c_WriteProtection(void)
{
    TXZ_Result result = TXZ_SUCCESS;
    /*  not support */
    return (result);	
}

/*--------------------------------------------------*/
/**
  * @brief      EEPROM_i2c_IsDeviceReady
  * @param      -
  * @retval     1     :Ready.
  * @retval     0     :Not Ready.
  */
/*--------------------------------------------------*/
int32_t EEPROM_i2c_IsDeviceReady(void)
{
    /*  not support */
    return (1);
}

/**
 *  @}
 */ /* End of group I2C_EEPROM_Exported_functions */

/**
 *  @}
 */ /* End of group I2C_EEPROM */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */

