/** 
 *******************************************************************************
 * @file    main.c
 * @brief   RMC UART Sample Application.
 * @version V1.0.0.1
 * $Date:: 2018-04-02 19:31:41 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include "txz_sample_def.h"
#include "bsp.h"
#include "bsp_uart_io.h"
#include "bsp_timer.h"
#include "remote.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup RMC_UART RMC_UART Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup RMC_UART_Private_macro RMC_UART Private Macro
 *  @{
 */

#define RMC_UART_CFG_NEC_FORMAT       (0)                         /* NEC Format     */
#define RMC_UART_CFG_TOSHIBA_FORMAT   (1)                         /* TOSHIBA Format */
#define RMC_UART_CFG_AEHA_FORMAT      (2)                         /* AEHA Format    */
#define RMC_UART_CFG_FORMAT       (RMC_UART_CFG_AEHA_FORMAT)

#define RLMLOSCCR_POSCEN_MASK              ((uint8_t)0x10)    /*!< POSCEN :[R/W]  :Mask     */
#define RLMLOSCCR_POSCEN_RW_DISABLE     ((uint8_t)0x00)    /*!< POSCEN :[R/W]  :Disable  */
#define RLMLOSCCR_POSCEN_RW_ENABLE      ((uint8_t)0x10)    /*!< POSCEN :[R/W]  :Enablc   */

#define RLMLOSCCR_POSCF_MASK             ((uint8_t)0x20)    /*!< POSCF  :[R]    :Mask     */
#define RLMLOSCCR_POSCF_STOP             ((uint8_t)0x00)    /*!< POSCF  :[R]    :Stop     */
#define RLMLOSCCR_POSCF_RUNNING        ((uint8_t)0x20)    /*!< POSCF  :[R]    :Running  */

#define LTTMR_VAL                       ((uint32_t)304)     /*!< 10MHz / (304+1) = 32.7869kHz */
/** 
 *  @}
 */ /* End of group RMC_UART_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup RMC_UART_Private_define RMC_UART Private Define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group RMC_UART_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup RMC_UART_Private_define RMC_UART Private Define
 *  @{
 */

#define MAIN_NULL       ((void *)0)     /*!< NULL. */
#define USER_ID_REMOTE  ((uint32_t)0)   /*!< User ID :Remote Control (Any value) */
#define USER_ID_UART    ((uint32_t)1)   /*!< User ID :USB UART (Any value) */

/** 
 *  @}
 */ /* End of group RMC_UART_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup RMC_UART_Private_define RMC_UART Private Define
 *  @{
 */
/*----------------------------------*/
/** 
 * @enum   Processing
 * @brief  Processing mode.
*/
/*----------------------------------*/
typedef enum
{
    PROC_IDLE,          /*!< Idle.      */
    PROC_START,         /*!< Start.     */
    PROC_STOP,          /*!< Stop.      */
    PROC_RUN,           /*!< Running.   */
    PROC_ERROR          /*!< Error.     */
} Processing;
/** 
 *  @}
 */ /* End of group RMC_UART_Private_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup RMC_UART_Private_typedef RMC_UART Private Typedef
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief Instance Information.
*/
/*----------------------------------*/
typedef struct
{
    t32a_t      t32a;              /*!< Driver      :T32A.     */
    timer_t     timer;             /*!< Application :Timer (1ms timer). */
    remote_t    *p_remote;         /*!< Application :Remote Control.  */
} instance_t;
/**
 *  @}
 */ /* End of group RMC_UART_Private_typedef */


/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup RMC_UART_Private_variables RMC_UART Private Variables
 *  @{
 */
static instance_t instance; /*!< Instance.                    */
static Processing proc;     /*!< Processing mode.             */
/**
 *  @}
 */ /* End of group RMC_UART_Private_variables */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup RMC_UART_Private_fuctions RMC_UART Private Fuctions
 *  @{
 */
static void variable_initialize(void);
static TXZ_Result driver_initialize(void);
static TXZ_Result driver_finalize(void);
static TXZ_Result application_initialize(void);
static TXZ_Result application_finalize(void);
static void timer_interval_handler(uint32_t id);
static void receive_data_handler(uint32_t id);
static TXZ_Result print_code(uint8_t src);
static TXZ_Result print_aeha_code(uint8_t src);
#ifdef USE_TBxOUT
  static void internal_hosc2_enable(void);
#endif
/*--------------------------------------------------*/
/** 
  * @brief  Variable Initialize
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void variable_initialize(void)
{
    /* Instance. */
    instance.t32a.p_instance    = MAIN_NULL;
    instance.timer.init.p_t32a  = MAIN_NULL;
    instance.p_remote           = MAIN_NULL;
    /* Proccesing. */
    proc = PROC_START;
}

/*--------------------------------------------------*/
/** 
  * @brief  Driver Initialize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result driver_initialize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* GPIO                 */
    /*----------------------*/
    /* Driver initialize has done within BSP. */
    /*----------------------*/
    /* T32A                 */
    /*----------------------*/
    {
        t32a_t *p_t32a = &instance.t32a;

        /*--- Construct  ---*/
        /* Register Allocation */
        switch (bsp_get_timer_ch(BSP_TIMER_1MS))
        {
        case 0:
            p_t32a->p_instance = TSB_T32A0;
            break;
        case 1:
            p_t32a->p_instance = TSB_T32A1;
            break;
        case 2:
            p_t32a->p_instance = TSB_T32A2;
            break;
        case 3:
            p_t32a->p_instance = TSB_T32A3;
            break;
        case 4:
            p_t32a->p_instance = TSB_T32A4;
            break;
        case 5:
            p_t32a->p_instance = TSB_T32A5;
            break;
        default:
            p_t32a->p_instance = MAIN_NULL;
            break;
        }
        /*--- Initialize ---*/
        p_t32a->init_mode.mode.halt = T32A_DBG_HALT_STOP;
        p_t32a->init_mode.mode.mode = T32A_MODE_16;
        if (p_t32a->p_instance != MAIN_NULL)
        {
            if (t32a_mode_init(p_t32a) != TXZ_SUCCESS)
            {
                result = TXZ_ERROR;
            }
        }
        else
        {
            result = TXZ_ERROR;
        }
    }
    /*----------------------*/
    /* Uart                 */
    /*----------------------*/
    /* Driver initialize will process within USB UART. */

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Driver Finalize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result driver_finalize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* Uart                 */
    /*----------------------*/
    /* Driver finalize has done within USB UART.    */
    /*----------------------*/
    /* T32A                 */
    /*----------------------*/
    {
        t32a_t *p_t32a = &instance.t32a;

        /*--- Finalize ---*/
        /* no processing */
        /*--- Destruct ---*/
        /* Register Release */
        p_t32a->p_instance = MAIN_NULL;
    }
    /*----------------------*/
    /* GPIO                 */
    /*----------------------*/
    /* Driver finalize will process within BSP.     */

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Application Initialize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_initialize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* Timer                */
    /*----------------------*/
    /* 1ms timer */
    {
        timer_t *p_timer = &instance.timer;

        p_timer->init.id       = (uint32_t)&instance.timer;
        p_timer->init.p_t32a   = &instance.t32a;
        p_timer->init.type     = bsp_get_timer_type(BSP_TIMER_1MS);
        p_timer->init.irq      = bsp_get_timer_nvic(BSP_TIMER_1MS);
        p_timer->init.interval = 1000;
        p_timer->init.handler  = timer_interval_handler;
        timer_initialize(p_timer);
    }
    /*----------------------*/
    /* Usb Uart             */
    /*----------------------*/
    /* Tx & Rx use same UART channel. */
    {
        uart_io_initial_setting_t param;

        param.id         = USER_ID_UART;
        switch (bsp_get_usb_uart_tx_ch(BSP_USB_UART_0))
        {
        case 0:
            param.p_reg  = TSB_UART0;
            break;
        case 1:
            param.p_reg  = TSB_UART1;
            break;
        case 2:
            param.p_reg  = TSB_UART2;
            break;
        default:
            param.p_reg  = MAIN_NULL;
            break;
        }
        param.boudrate   = bsp_get_usb_uart_tx_baudrate(BSP_USB_UART_0);
        param.irq.tx     = bsp_get_usb_uart_tx_nvic(BSP_USB_UART_0);
        param.irq.rx     = bsp_get_usb_uart_rx_nvic(BSP_USB_UART_0);
        param.irq.err    = bsp_get_usb_uart_err_nvic(BSP_USB_UART_0);
        if (param.p_reg != MAIN_NULL)
        {
            result  = uart_io_initialize(&param);
        }
        else
        {
            result = TXZ_ERROR;
        }
    }
    /*----------------------*/
    /* Remote Control       */
    /*----------------------*/
    {
        remote_initial_setting_t param;

        param.id            = USER_ID_REMOTE;
#if (RMC_UART_CFG_FORMAT == RMC_UART_CFG_NEC_FORMAT)
        param.format        = REMOTE_FMT_TYPE_NEC;
#elif (RMC_UART_CFG_FORMAT == RMC_UART_CFG_TOSHIBA_FORMAT)
        param.format        = REMOTE_FMT_TYPE_TOSHIBA;
#elif (RMC_UART_CFG_FORMAT == RMC_UART_CFG_AEHA_FORMAT)
        param.format        = REMOTE_FMT_TYPE_AEHA;
#endif
        param.receive_data  = receive_data_handler;
        instance.p_remote = remote_initialize(&param);
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Application Finalize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_finalize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* Remote Control       */
    /*----------------------*/
    if (instance.p_remote != MAIN_NULL)
    {
        remote_finalize(instance.p_remote);
    }
    /*----------------------*/
    /* Usb Uart             */
    /*----------------------*/
    {
        uart_io_finalize();
    }
    /*----------------------*/
    /* Timer                */
    /*----------------------*/
    if (instance.timer.init.p_t32a != MAIN_NULL)
    {
        timer_finalize(&instance.timer);
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Interval Time Handler.
  * @param  id        :User ID
  * @return -
  * @retval -
  * @note   for 1ms Timer.
  * @note   Call by Timer.
  */
/*--------------------------------------------------*/
static void timer_interval_handler(uint32_t id)
{
    if (id == (uint32_t)&instance.timer)
    {
        /* Increment Tick. Tick is used to driver's Timeout. */
        hal_inc_tick();
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  Receive Data Handler.
  * @param  id        :User ID
  * @return -
  * @retval -
  * @note   Call by Remote Control.
  */
/*--------------------------------------------------*/
static void receive_data_handler(uint32_t id)
{
    if (id == USER_ID_REMOTE)
    {
        /* Sample Soft don't use. */
    }
}
/*--------------------------------------------------*/
/** 
  * @brief  Print Code.
  * @param  src   :Output Data.
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result print_code(uint8_t src)
{
    TXZ_Result res = TXZ_SUCCESS;
    int32_t i = 0;

    if (printf("=>") < 0)
    {
        res = TXZ_ERROR;
    }
    for (i=7; i>=0; i--)
    {
        uint8_t dst = (uint8_t)((src & (uint8_t)(1 << i)) >> i);
        if (printf("%1d", dst) < 0)
        {
            res = TXZ_ERROR;
        }
    } 
    if (printf(":0x%02X  \n", src) < 0)
    {
        res = TXZ_ERROR;
    }

    return (res);
}
/*--------------------------------------------------*/
/** 
  * @brief  Print AEHA Code.
  * @param  src   :Output Data.
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result print_aeha_code(uint8_t src)
{
    TXZ_Result res = TXZ_SUCCESS;
    int32_t i = 0;
    uint8_t tmp = 0;
    uint8_t tmp_l = 0;
    uint8_t tmp_h = 0;

    if (printf("=>") < 0)
    {
        res = TXZ_ERROR;
    }
    for (i=7; i>=0; i--)
    {
        uint8_t dst = (uint8_t)((src & (uint8_t)(1 << i)) >> i);
        tmp = tmp + (dst << (7- i));
        if (printf("%1d", dst) < 0)
        {
            res = TXZ_ERROR;
        }
    } 
    tmp_l = tmp & 0x0FU;
    tmp_h = tmp & 0xF0U;
    tmp = (tmp_l << 4) + (tmp_h >> 4);
    if (printf(":0x%02X  \n", tmp) < 0)
    {
        res = TXZ_ERROR;
    }

    return (res);
}
#ifdef USE_TBxOUT
/*--------------------------------------------------*/
/** 
  * @brief  Internal HOSC Enable
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void internal_hosc2_enable(void)
{
    uint8_t work;

    work = (uint8_t)(TSB_RLM->LOSCCR & ~RLMLOSCCR_POSCEN_MASK);
    TSB_RLM->LOSCCR = (uint8_t)(work | RLMLOSCCR_POSCEN_RW_ENABLE);
    while ((TSB_RLM->LOSCCR & RLMLOSCCR_POSCF_MASK) != RLMLOSCCR_POSCF_RUNNING) 
    {
        /* no processing */
    }
}
#endif
/**
 *  @}
 */ /* End of group RMC_UART_Private_fuctions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup RMC_UART_Exported_functions RMC_UART Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    /*----------------------*/
    /* BSP initialize       */
    /*----------------------*/
    bsp_initialize();
  #ifdef USE_TBxOUT
    internal_hosc2_enable();
    {
        TSB_LTT0->CR0 = 0x00;
        TSB_LTT0->VALL = LTTMR_VAL & 0xFF;
        TSB_LTT0->VALH = (LTTMR_VAL >> 8)& 0xFF;;
        TSB_LTT0->CR0 = 0x02;
    }
  #endif
    /*----------------------*/
    /* Initialize           */
    /*----------------------*/
    /*--- variable    ---*/
    variable_initialize();
    /*--- driver      ---*/
    if (driver_initialize() != TXZ_SUCCESS)
    {
        proc = PROC_ERROR;
    }
    /*--- application ---*/
    if (application_initialize() != TXZ_SUCCESS)
    {
        proc = PROC_ERROR;
    }
    /*----------------------*/
    /* Main Processing      */
    /*----------------------*/
    while(proc != PROC_ERROR)
    {
        switch(proc)
        {
        /*--- Start ---*/
        case PROC_START:
            {
                timer_start(&instance.timer);
                remote_start(instance.p_remote);
                proc = PROC_RUN;
            }
            break;
        /*--- Stop ---*/
        case PROC_STOP:
            {
                remote_stop(instance.p_remote);
                timer_stop(&instance.timer);
                /* error */
                proc = PROC_ERROR;
            }
            break;
        /*--- Receive Data & Output Code. ---*/
        case PROC_RUN:
            {
                remote_data_t data;
                int32_t i =0;
                if(instance.p_remote->init.format == REMOTE_FMT_TYPE_AEHA)
                {
                    for(i=0; i<10; i++)
                    {
                        data.aeha.data[i]  = 0;
                    }
                }
                /* Get the Rx Data. */
                if (remote_get_list_data(instance.p_remote, &data) == TXZ_SUCCESS)
                {
                    if ((instance.p_remote->init.format == REMOTE_FMT_TYPE_NEC) || (instance.p_remote->init.format == REMOTE_FMT_TYPE_TOSHIBA))
                    {
                        /* Output :Customer Code */
                        if (printf("Customer Code \n") < 0)
                        {
                            proc = PROC_STOP;
                        }
                        if (print_code((uint8_t)((uint16_t)(data.sample.customer & 0xFF00) >> 8)) != TXZ_SUCCESS)
                        {
                            proc = PROC_STOP;
                        }
                        if (print_code((uint8_t)(data.sample.customer & 0x00FF)) != TXZ_SUCCESS)
                        {
                            proc = PROC_STOP;
                        }
                        /* Output :Data Code */
                        if (printf("Data Code \n") < 0)
                        {
                            proc = PROC_STOP;
                        }
                        if (print_code(data.sample.data) != TXZ_SUCCESS)
                        {
                            proc = PROC_STOP;
                        }
                        if (print_code(data.sample.rdata) != TXZ_SUCCESS)
                        {
                            proc = PROC_STOP;
                        }
                    }else if(instance.p_remote->init.format == REMOTE_FMT_TYPE_AEHA)
                    {
                        uint32_t    length = data.aeha.bit;
                        if(length > 72)
                        {
                            length = 72;
                        }
                        /* Output :Customer Code */
                        if (printf("Remocon Code \n") < 0)
                        {
                            proc = PROC_STOP;
                        }
                        if (print_aeha_code(data.aeha.customer[0]) != TXZ_SUCCESS)
                        {
                            proc = PROC_STOP;
                        }
                        if(length > 8)
                        {
                            length -= 8;
                            if (print_aeha_code(data.aeha.customer[1]) != TXZ_SUCCESS)
                            {
                                proc = PROC_STOP;
                            }
                        }
                        i =0;
                        while (length > 8)
                        {
                            if (print_aeha_code(data.aeha.data[i]) != TXZ_SUCCESS)
                            {
                                proc = PROC_STOP;
                            }
                            i++;
                            length -= 8;
                        }
                    }
                }
            }
            break;
        /*--- Error ---*/
        case PROC_ERROR:
            /* no processing */
            break;
        /*--- Default ---*/
        default:
            proc = PROC_ERROR;
            break;
        }
    }
    /*>>> System Error. Please Debug. */
    /*----------------------*/
    /* Finalize             */
    /*----------------------*/
    /*--- application ---*/
    if (application_finalize() != TXZ_SUCCESS)
    {
        /* no processing */
    }
    /*--- driver      ---*/
    if (driver_finalize() != TXZ_SUCCESS)
    {
        /* no processing */
    }
    /*----------------------*/
    /* BSP finalize         */
    /*----------------------*/
    bsp_finalize();
#ifdef DEBUG
    assert_failed(MAIN_NULL, 0);
#endif

    return(0);
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART reception IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_rx(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_rx_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }

}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART transmit IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_tx(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_tx_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART error IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_err(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_err_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  Timer.
  * @param  timer :Timer.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_timer(BSPTimer timer)
{
    switch (timer)
    {
    case BSP_TIMER_1MS:
        timer_irq_handler(&instance.timer);
        break;
    default:
        /* no processing */
        break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  RMC IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_rmc(uint32_t status)
{
    remote_irq_handler(instance.p_remote, status);
}

#ifdef DEBUG
/*--------------------------------------------------*/
/** 
  * @brief  Failure Assert.
  * @note   for debug
  */
/*--------------------------------------------------*/
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __nop();
    }
}
#endif

/**
 *  @}
 */ /* End of group RMC_UART_Exported_functions */

/**
 *  @}
 */ /* End of group RMC_UART */

/**
 *  @} 
 */ /* End of group Example */


#ifdef __cplusplus
}
#endif /* __cplusplus */
