/**
 *******************************************************************************
 * @file    remote.c
 * @brief   This file provides API functions for Remote Control.
 * @version V1.0.0.1
 * $Date:: 2018-04-02 19:31:41 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif
/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "remote.h"
#include "bsp.h"

#if defined(__REMOTE_H)

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @addtogroup RMC_UART
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup RMC_UART_Private_macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group RMC_UART_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup RMC_UART_Private_define
 *  @{
 */

/*--- for Debug ---*/
#ifdef DEBUG
    #if 1
        #define REMOTE_CFG_DEBUG
    #endif
#endif
/** 
 *  @}
 */ /* End of group RMC_UART_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup RMC_UART_Private_define
 *  @{
 */

/** 
 *  @defgroup NULL_Pointer Null Pointer
 *  @{
 */
#define REMOTE_NULL                ((void *)0)
/**
 *  @}
 */ /* End of defgroup NULL_Pointer */

/** 
 *  @defgroup FormatSampleDefinition Format Sample Definition
 *  @brief Format Type = Sample.
 *  @{
 */
#define FMT_SAMPLE_CUSTOM_BIT_NUM   (16UL)      /*!< Num of bits :Custom Code        */
#define FMT_SAMPLE_DATA_BIT_NUM     (8UL)       /*!< Num of bits :Data Code          */
#define FMT_SAMPLE_R_DATA_BIT_NUM   (8UL)       /*!< Num of bits :Reversed Data Code */
#define FMT_SAMPLE_TOTAL_BIT_NUM    ((uint32_t)(FMT_SAMPLE_CUSTOM_BIT_NUM + FMT_SAMPLE_DATA_BIT_NUM + FMT_SAMPLE_R_DATA_BIT_NUM))  /*!< Num of bits :Total */
/**
 *  @}
 */ /* End of name FormatSampleDefinition */

/** 
 *  @}
 */ /* End of group RMC_UART_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup RMC_UART_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group RMC_UART_Private_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup RMC_UART_Private_typedef
 *  @{
 */
 /*
 *  @defgroup RMC_LeaderInterruptFlag  RMC leader detection interrupt flag.
 *  @brief    Leader detection interrupt flag.
 *  @{
 */
#define RMC_LD_INT_FLAG_OFF         ((uint32_t)0x00000000)    /*!< It doesn't occur. */
#define RMC_LD_INT_FLAG_ON          ((uint32_t)0x00000001)    /*!< It occurs.        */
/**
 *  @}
 */ /* End of group RMC_LeaderInterruptFlag */
/**
 *  @}
 */ /* End of group RMC_UART_Private_typedef */


/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup RMC_UART_Private_variables
 *  @{
 */

static remote_t instance;    /*!< Instance. */
static uint32_t ld_int_flg = RMC_LD_INT_FLAG_OFF;

/**
 *  @}
 */ /* End of group RMC_UART_Private_variables */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup RMC_UART_Private_fuctions
 *  @{
 */
static void clear_rx_list(remote_private_rx_list_t *p_list);
static void debug_error(void);
/*--------------------------------------------------*/
/** 
  * @brief  Error Hook
  * @param  -
  * @retval -
  * @note   for debug
  */
/*--------------------------------------------------*/
static void debug_error(void)
{
#ifdef REMOTE_CFG_DEBUG
    while(1)
    {
        __nop();
    }
#endif
}

/*--------------------------------------------------*/
/** 
  * @brief  Clear Receive Data List.
  * @param  p_list :List Source Address.
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void clear_rx_list(remote_private_rx_list_t *p_list)
{
    p_list->num = 0;
    p_list->wp  = 0;
    p_list->rp  = 0;
}

/**
 *  @}
 */ /* End of group RMC_UART_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup RMC_UART_Exported_functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Initialize.
  * @param  p_param :Initial Setting Information Source Address.
  * @return Instance Address.
  * @retval (value != NULL) :Valid value.
  * @note   Parameter check isn't performed.
  * @attention Singleton.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
remote_t *remote_initialize(remote_initial_setting_t *p_param)
{
    rmc_t *p_rmc = &instance.info.rmc;

    /*------------------------------*/
    /*  Initial Parameter Copy      */
    /*------------------------------*/
    instance.init.id            = p_param->id;
    instance.init.format        = p_param->format;
    instance.init.receive_data  = p_param->receive_data;
    /*------------------------------*/
    /*  Clear Private Information   */
    /*------------------------------*/
    clear_rx_list(&instance.info.list);
    /*------------------------------*/
    /*  Construct                   */
    /*------------------------------*/
    /* Register Allocation */
    p_rmc->p_instance = TSB_RMC0;
    /*------------------------------*/
    /*  Initialize                  */
    /*------------------------------*/
    /*--- Format Type = Smaple ---*/
    if (instance.init.format == REMOTE_FMT_TYPE_NEC)
    {
      #ifndef USE_TBxOUT
        /* Select source clock. "fs" */
        p_rmc->init.fssel.clk  = RMC_CLK_LOW_SPEED;         /* low speed     :32.768(kHz) */
      #else
        /* Select source clock. "TBxOUT" */
        p_rmc->init.fssel.clk  = RMC_CLK_TIMER;             /* TBxOUT        :32.7869(kHz) */
      #endif
        /* Receive Control Setting "1". */
        p_rmc->init.cnt1.lcMax = (uint32_t)0x70;            /* Period Max    :13.7(msec)  */
        p_rmc->init.cnt1.lcMin = (uint32_t)0x6C;            /* Period Min    :13.3(msec)  */
        p_rmc->init.cnt1.llMax = (uint32_t)0x4B;            /* Low Width Max : 9.2(msec)  */
        p_rmc->init.cnt1.llMin = (uint32_t)0x48;            /* Low Width Min : 8.8(msec)  */
        /* Receive Control Setting "2". */
        p_rmc->init.cnt2.lien  = RMC_LI_ENABLE;             /* Enable                     */
        p_rmc->init.cnt2.edien = RMC_EDI_DISABLE;           /* Disable                    */
        p_rmc->init.cnt2.cld   = RMC_LD_DISABLE;            /* Disable                    */
        p_rmc->init.cnt2.phim  = RMC_PHM_DISABLE;           /* Disable                    */
        p_rmc->init.cnt2.ll    = RMC_LL_MAX;                /* Disable                    */
        p_rmc->init.cnt2.dmax  = (uint32_t)0xFE;            /* Received End  : 7.8(msec)  */
        /* Receive Control Setting "3". */
        p_rmc->init.cnt3.dath  = RMC_THRESH_HIGH_MIN;       /* Disable                    */
        p_rmc->init.cnt3.datl  = (uint32_t)0x37;            /* Threshold     : 1.7(msec)  */
        /* Receive Control Setting "4". */
        p_rmc->init.cnt4.po    = RMC_POLARITY_POSITIVE;     /* Positive                  */
        p_rmc->init.cnt4.nc    = RMC_NOISE_REDUCTION_MIN;   /* Disable                   */
        /* Receive End Bit1 setting. */
        p_rmc->init.end1.end1  = RMC_RX_DATA_BITS_MIN;      /* Disable                   */
        /* Receive End Bit2 setting. */
        p_rmc->init.end2.end2  = RMC_RX_DATA_BITS_MIN;      /* Disable                   */
        /* Receive End Bit3 setting. */
        p_rmc->init.end3.end3  = RMC_RX_DATA_BITS_MIN;      /* Disable                   */
    }
    else if (instance.init.format == REMOTE_FMT_TYPE_TOSHIBA)
    {
      #ifndef USE_TBxOUT
        /* Select source clock. "fs" */
        p_rmc->init.fssel.clk  = RMC_CLK_LOW_SPEED;         /* low speed     :32.768(kHz) */
      #else
        /* Select source clock. "TBxOUT" */
        p_rmc->init.fssel.clk  = RMC_CLK_TIMER;             /* TBxOUT        :32.7869(kHz) */
      #endif
        /* Receive Control Setting "1". */
        p_rmc->init.cnt1.lcMax = (uint32_t)0x4B;            /* Period Max    : 9.2(msec)  */
        p_rmc->init.cnt1.lcMin = (uint32_t)0x48;            /* Period Min    : 8.8(msec)  */
        p_rmc->init.cnt1.llMax = (uint32_t)0x26;            /* Low Width Max : 4.7(msec)  */
        p_rmc->init.cnt1.llMin = (uint32_t)0x23;            /* Low Width Min : 4.3(msec)  */
        /* Receive Control Setting "2". */
        p_rmc->init.cnt2.lien  = RMC_LI_ENABLE;             /* Enable                     */
        p_rmc->init.cnt2.edien = RMC_EDI_DISABLE;           /* Disable                    */
        p_rmc->init.cnt2.cld   = RMC_LD_DISABLE;            /* Disable                    */
        p_rmc->init.cnt2.phim  = RMC_PHM_DISABLE;           /* Disable                    */
        p_rmc->init.cnt2.ll    = RMC_LL_MAX;                /* Disable                    */
        p_rmc->init.cnt2.dmax  = (uint32_t)0xFE;            /* Received End  : 7.8(msec)  */
        /* Receive Control Setting "3". */
        p_rmc->init.cnt3.dath  = RMC_THRESH_HIGH_MIN;       /* Disable                    */
        p_rmc->init.cnt3.datl  = (uint32_t)0x37;            /* Threshold     : 1.7(msec)  */
        /* Receive Control Setting "4". */
        p_rmc->init.cnt4.po    = RMC_POLARITY_POSITIVE;     /* Positive                  */
        p_rmc->init.cnt4.nc    = RMC_NOISE_REDUCTION_MIN;   /* Disable                   */
        /* Receive End Bit1 setting. */
        p_rmc->init.end1.end1  = RMC_RX_DATA_BITS_MIN;      /* Disable                   */
        /* Receive End Bit2 setting. */
        p_rmc->init.end2.end2  = RMC_RX_DATA_BITS_MIN;      /* Disable                   */
        /* Receive End Bit3 setting. */
        p_rmc->init.end3.end3  = RMC_RX_DATA_BITS_MIN;      /* Disable                   */
    }
    else if (instance.init.format == REMOTE_FMT_TYPE_AEHA)
    {
      #ifndef USE_TBxOUT
        /* Select source clock. "fs" */
        p_rmc->init.fssel.clk  = RMC_CLK_LOW_SPEED;         /* low speed     :32.768(kHz) */
      #else
        /* Select source clock. "TBxOUT" */
        p_rmc->init.fssel.clk  = RMC_CLK_TIMER;             /* TBxOUT        :32.7869(kHz) */
      #endif
        /* Receive Control Setting "1". */
        p_rmc->init.cnt1.lcMax = (uint32_t)0x2C;            /* Period Max    : 5.4(msec)  */
        p_rmc->init.cnt1.lcMin = (uint32_t)0x28;            /* Period Min    : 4.9(msec)  */
        p_rmc->init.cnt1.llMax = (uint32_t)0x1E;            /* Low Width Max : 3.7(msec)  */
        p_rmc->init.cnt1.llMin = (uint32_t)0x1A;            /* Low Width Min : 3.2(msec)  */
        /* Receive Control Setting "2". */
        p_rmc->init.cnt2.lien  = RMC_LI_ENABLE;             /* Enable                     */
        p_rmc->init.cnt2.edien = RMC_EDI_DISABLE;           /* Disable                    */
        p_rmc->init.cnt2.cld   = RMC_LD_DISABLE;            /* Disable                    */
        p_rmc->init.cnt2.phim  = RMC_PHM_DISABLE;           /* Disable                    */
        p_rmc->init.cnt2.ll    = RMC_LL_MAX;                /* Disable                    */
        p_rmc->init.cnt2.dmax  = (uint32_t)0xFE;            /* Received End  : 7.8(msec)  */
        /* Receive Control Setting "3". */
        p_rmc->init.cnt3.dath  = RMC_THRESH_HIGH_MIN;       /* Disable                    */
        p_rmc->init.cnt3.datl  = (uint32_t)0x28;            /* Threshold     : 1.2(msec)  */
        /* Receive Control Setting "4". */
        p_rmc->init.cnt4.po    = RMC_POLARITY_POSITIVE;     /* Positive                  */
        p_rmc->init.cnt4.nc    = RMC_NOISE_REDUCTION_MIN;   /* Disable                   */
        /* Receive End Bit1 setting. */
        p_rmc->init.end1.end1  = RMC_RX_DATA_BITS_MIN;      /* Disable                   */
        /* Receive End Bit2 setting. */
        p_rmc->init.end2.end2  = RMC_RX_DATA_BITS_MIN;      /* Disable                   */
        /* Receive End Bit3 setting. */
        p_rmc->init.end3.end3  = RMC_RX_DATA_BITS_MIN;      /* Disable                   */
    }
    /*--- Other Type :Not Support ---*/
    else
    {
        debug_error();
    }
    /* initialize */
    if (rmc_init(p_rmc) != TXZ_SUCCESS)
    {
        debug_error();
    }

    return(&instance);
}

/*--------------------------------------------------*/
/** 
  * @brief  Finalize.
  * @param  p_instance :Instance Source Address.
  * @return -
  * @retval -
  * @note   Parameter check isn't performed.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void remote_finalize(remote_t *p_instance)
{
    rmc_t *p_rmc = &p_instance->info.rmc;

    /*------------------------------*/
    /*  Stop                        */
    /*------------------------------*/
    remote_stop(p_instance);
    /*------------------------------*/
    /*  Finalize                    */
    /*------------------------------*/
    if (rmc_deinit(p_rmc) != TXZ_SUCCESS)
    {
        debug_error();
    }
    /*------------------------------*/
    /*  Destruct                    */
    /*------------------------------*/
    /* Register Release */
    p_rmc->p_instance = REMOTE_NULL;
}

/*--------------------------------------------------*/
/** 
  * @brief  Start to receive.
  * @param  p_instance :Instance Source Address.
  * @retval -
  * @note   Parameter check isn't performed.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void remote_start(remote_t *p_instance)
{
    rmc_t *p_rmc = &p_instance->info.rmc;

    /*--- RMC    ---*/
    if (rmc_rx_enable(p_rmc) != TXZ_SUCCESS)
    {
        debug_error();
    }
    /*--- INTRMC ---*/
    bsp_irq_rmc_enable();
    /*--- NVIC   ---*/
    NVIC_EnableIRQ(INTRMC0_IRQn);
}

/*--------------------------------------------------*/
/** 
  * @brief  Stop to receive.
  * @param  p_instance :Instance Source Address.
  * @retval -
  * @note   Parameter check isn't performed.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void remote_stop(remote_t *p_instance)
{
    rmc_t *p_rmc = &p_instance->info.rmc;

    /*--- NVIC   ---*/
    NVIC_DisableIRQ(INTRMC0_IRQn);
    /*--- RMC    ---*/
    if (rmc_rx_disable(p_rmc) != TXZ_SUCCESS)
    {
        debug_error();
    }
    /*--- INTRMC ---*/
    bsp_irq_rmc_disable();
}

/*--------------------------------------------------*/
/** 
  * @brief  Get Receive Data.
  * @param  p_instance :Instance Source Address.
  * @param  p_data     :Receive Data Destination Address.
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :List Empty. or \n
                        :Not Support Format Type.
  * @note   Parameter check isn't performed.
  * @attention This function is not available in interrupt.
  * @attention In progress, RMC Interrupt Disable.
  */
/*--------------------------------------------------*/
TXZ_Result remote_get_list_data(remote_t *p_instance, remote_data_t *p_data)
{
    TXZ_Result res = TXZ_ERROR;
    if (p_instance->info.list.num != 0)
    {
        remote_private_rx_info_t *p_info = &p_instance->info.list.info[p_instance->info.list.rp];

        /*------------------------------*/
        /*  Get Data                    */
        /*------------------------------*/
        /*--- Format Type = Smaple ---*/
        if ((instance.init.format == REMOTE_FMT_TYPE_NEC) || (instance.init.format == REMOTE_FMT_TYPE_TOSHIBA))
        {
            if (p_info->bits == FMT_SAMPLE_TOTAL_BIT_NUM)
            {
                p_data->sample.customer = (uint16_t)((p_info->data[0] & 0xFFFF0000) >> 16);
                p_data->sample.data     = (uint8_t)((p_info->data[0] & 0x0000FF00) >> 8);
                p_data->sample.rdata    = (uint8_t)(p_info->data[0] & 0x000000FF);
                /* Success !! */
                res = TXZ_SUCCESS;
            }
        }
        else if (instance.init.format == REMOTE_FMT_TYPE_AEHA)
        {
                uint32_t length = 0;
                uint32_t num = 0;
                uint32_t sf = 0;
                uint32_t i = 0;
                uint32_t mask;
                
                p_data->aeha.bit = p_info->bits;
                
                length = p_data->aeha.bit;
                if(length > 71)
                {
                    length = 71;
                }
                i = 0;
                while((length > 0) && (i<2))
                {
                    num = (length-1) / 32;
                    
                    mask = 0x000000FF;
                    sf = length % 32;
                    if(sf ==  0) sf = 32;
                    if(sf < 8)
                    {
                        uint8_t temp;
                        
                        mask >>= 8 - sf;
                        temp = (uint8_t)((p_info->data[num] & mask) << (8-sf));
                        if((length > 32) && (num > 0))
                        {
                            mask = 0xFF000000;
                            mask <<= sf;
                            temp |= (uint8_t)((p_info->data[num-1] & mask) >> (24+sf));
                        }
                        p_data->aeha.customer[i] = temp;
                    }
                    else{
                        sf -= 8;
                        mask <<= sf;
                        p_data->aeha.customer[i] = (uint8_t)((p_info->data[num] & mask) >> sf);
                    }
                    if(length < 8){
                        length = 0;
                    }
                    else
                    {
                        length -= 8;
                    }
                    ++i;
                }
                i = 0;
                while((length > 0) && (i<10))
                {
                    num = (length-1) / 32;
                    
                    mask = 0x000000FF;
                    sf = length % 32;
                    if(sf ==  0) sf = 32;
                    if(sf < 8)
                    {
                        uint8_t temp;
                        
                        mask >>= 8 - sf;
                        temp = (uint8_t)((p_info->data[num] & mask) << (8-sf));
                        if((length > 32) && (num > 0))
                        {
                            mask = 0xFF000000;
                            mask <<= sf;
                            temp |= (uint8_t)((p_info->data[num-1] & mask) >> (24+sf));
                        }
                        p_data->aeha.data[i] = temp;
                    }
                    else{
                        sf -= 8;
                        mask <<= sf;
                        p_data->aeha.data[i] = (uint8_t)((p_info->data[num] & mask) >> sf);
                    }
                    if(length < 8){
                        length = 0;
                    }
                    else
                    {
                        length -= 8;
                    }
                    ++i;
                }
                /* Success !! */
                res = TXZ_SUCCESS;
        }
        /*------------------------------*/
        /*  Update List                 */
        /*------------------------------*/
        /*--- Read Pointer ---*/
        p_instance->info.list.rp++;
        if (p_instance->info.list.rp >= REMOTE_RX_LIST_NUM_MAX)
        {
            p_instance->info.list.rp = 0;
        }
        /*--- List Num ---*/
        /*>>> Critical information. */
        NVIC_DisableIRQ(INTRMC0_IRQn);
        p_instance->info.list.num -= 1;
        NVIC_EnableIRQ(INTRMC0_IRQn);
        /*<<< Critical information. */
    }

    return (res);
}

/*--------------------------------------------------*/
/** 
  * @brief  IRQ Handler.
  * @param  p_instance :Instance Source Address.
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void remote_irq_handler(remote_t *p_instance, uint32_t status)
{
    rmc_t *p_rmc = &p_instance->info.rmc;
    /*------------------------------*/
    /*  Check Status                */
    /*------------------------------*/
    /* Check to the leader detection interrupt flag. */
    if ((status & RMC_LD_INT_OCCUR) == RMC_LD_INT_OCCUR)
    {
        /* leader detection interrupt ON */
        ld_int_flg =  RMC_LD_INT_FLAG_ON;
    }
    /* Check to the data bit cycle max interrupt flag. */
    if (((status & RMC_CYCLE_MAX_INT_OCCUR) == RMC_CYCLE_MAX_INT_OCCUR) && 
        (ld_int_flg ==  RMC_LD_INT_FLAG_ON))
    {
     
        remote_private_rx_info_t *p_info = &p_instance->info.list.info[p_instance->info.list.wp];
        /* leader detection interrupt CLEAR */
        ld_int_flg =  RMC_LD_INT_FLAG_OFF;
        /*------------------------------*/
        /*  Update List                 */
        /*------------------------------*/
        /*--- Bits ---*/
        p_info->bits = (uint8_t)(status & RMC_RX_BIT_NUM_MASK);
        /*--- Data ---*/
        if (rmc_get_data(p_rmc, &p_info->data[0]) != TXZ_SUCCESS)
        {
            debug_error();
        }
        /*--- Write Pointer ---*/
        p_instance->info.list.wp++;
        if (p_instance->info.list.wp >= REMOTE_RX_LIST_NUM_MAX)
        {
             p_instance->info.list.wp = 0;
        }
        /*--- List Num ---*/
        /*>>> Critical information. */
        p_instance->info.list.num += 1;
        /*<<< Critical information. */
        /*------------------------------*/
        /*  Call Handler                */
        /*------------------------------*/
        if (p_instance->init.receive_data != REMOTE_NULL)
        {
            p_instance->init.receive_data(p_instance->init.id);
        }
    }
}

/**
 *  @}
 */ /* End of group RMC_UART_Exported_functions */

/**
 *  @}
 */ /* End of group RMC_UART */

/**
 *  @} 
 */ /* End of group Example */

#endif /* defined(__REMOTE_H)  */
#ifdef __cplusplus
}
#endif /* __cplusplus */
