/** 
 *******************************************************************************
 * @file    remote.h
 * @brief   This file provides all the functions prototypes for Remote Control.
 * @version V1.0.0.1
 * $Date:: 2017-10-04 17:24:08 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __REMOTE_H
#define __REMOTE_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_sample_def.h"
#include "rmc.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @addtogroup RMC_UART
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup RMC_UART_Exported_macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group RMC_UART_Exported_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup RMC_UART_Exported_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group RMC_UART_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup RMC_UART_Private_define
 *  @{
 */
#define REMOTE_RX_LIST_NUM_MAX          (8UL)       /*!< Max Num of Rx List. */
/** 
 *  @}
 */ /* End of group RMC_UART_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup RMC_UART_Exported_define
 *  @{
 */

/*----------------------------------*/
/** 
 * @brief  Format Type.
*/
/*----------------------------------*/
typedef enum {
    REMOTE_FMT_TYPE_NEC     = 0U,     /*!< NEC Format      */
    REMOTE_FMT_TYPE_TOSHIBA = 1U,     /*!< TOSHIBA Format  */
    REMOTE_FMT_TYPE_AEHA    = 2U      /*!< AEHA Format     */
} RemoteFormatType;

/** 
 *  @}
 */ /* End of group RMC_UART_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup RMC_UART_Private_typedef
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief  Receive Data Information.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t    data[RMC_DATA_BUFF_NUM]; /*!< Rx Data.        */
    uint8_t     bits;                    /*!< Num of Rx Bits. */
} remote_private_rx_info_t;

/*----------------------------------*/
/** 
 * @brief  Receive Data List.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t            num;    /*!< Num of RMC RX Data. */
    uint32_t            wp;     /*!< List Write Pointer. */
    uint32_t            rp;     /*!< List Read Pointer.  */
    remote_private_rx_info_t info[REMOTE_RX_LIST_NUM_MAX]; /*!< Receive Data Information. */
} remote_private_rx_list_t;

/*----------------------------------*/
/** 
 * @brief  Private Information.
*/
/*----------------------------------*/
typedef struct
{
    rmc_t  rmc;                      /*!< RMC Instance.      */
    remote_private_rx_list_t list;   /*!< Receive Data List.   */
} remote_private_info_t;

/**
 *  @}
 */ /* End of group RMC_UART_Private_typedef */


/** 
 *  @addtogroup RMC_UART_Exported_typedef
 *  @{
 */

/*----------------------------------*/
/** 
 * @brief  Remote data. \n
 *         for Format Type = Smaple.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t customer;              /*!< Customer Code.  */
    uint8_t  data;                  /*!< Data Code.      */
    uint8_t  rdata;                 /*!< Data Code (Reverse). */
} remote_data_fmt_sample_t;

/*----------------------------------*/
/** 
 * @brief  Remote data. \n
 *         for Format Type = Smaple.
*/
/*----------------------------------*/
typedef struct
{
    uint8_t bit;                    /*!< Num of Data bits */
    uint8_t  customer[2];              /*!< Data Code.      */
    uint8_t  data[10];              /*!< Data Code.      */
} remote_data_fmt_aeha_t;

/*----------------------------------*/
/** 
 * @brief  Remote data.
*/
/*----------------------------------*/
typedef union
{
    remote_data_fmt_sample_t sample;    /*!< Format Type :Smaple */
    remote_data_fmt_aeha_t   aeha;      /*!< Format Type :AEHA   */
} remote_data_t;

/*----------------------------------*/
/** 
 * @brief  Initial setting. \n
 *         User must be set.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t              id;           /*!< User ID.            */
    RemoteFormatType      format;       /*!< Format Type.        */
    void (*receive_data)(uint32_t id);  /*!< Event handler. Notify of receive data. */
} remote_initial_setting_t;

/*----------------------------------*/
/** 
 * @brief  Remote Control Class.
*/
/*----------------------------------*/
typedef struct
{
    remote_initial_setting_t init;   /*!< Initial setting.     */
    remote_private_info_t    info;   /*!< Private information. */
} remote_t;

/**
 *  @}
 */ /* End of group RMC_UART_Exported_typedef */


/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup RMC_UART_Exported_functions
 *  @{
 */
remote_t *remote_initialize(remote_initial_setting_t *p_param);
void remote_finalize(remote_t *p_instance);
void remote_start(remote_t *p_instance);
void remote_stop(remote_t *p_instance);
TXZ_Result remote_get_list_data(remote_t *p_instance, remote_data_t *p_data);
void remote_irq_handler(remote_t *p_instance, uint32_t status);
/**
 *  @}
 */ /* End of group RMC_UART_Exported_functions */

/**
 *  @}
 */ /* End of group RMC_UART */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __REMOTE_H */


